using System;
using System.IO;
using System.Collections;
using Genghis;
using clp = Genghis.CommandLineParser;

// NOTE: This would override the assembly's description
//[clp.ParserUsage("A test of the Genghis command line parser.")]
class TestCommandLine : CommandLineParser
{
    [clp.ValueUsage("Minimum x value.", ValueName = "min value")]
    public int  minX = 10;

    [clp.FlagUsage("Turn on debugger.", IgnoreCase = false)]
    public bool debug = false;

    [clp.FlagUsage("Register the server.")]
    public bool regserver = false;

    [clp.ValueUsage("Intensity value.", Optional = false)]
    public double   intensity = 0;

    // NOTE: Testing variables with no Usage
    public string   foo = "bar";

    // TODO: RestrictedParameterUsage
    //[clp.FlagUsage("File extension values are:\n\tbmp\n\tgif\n\tjpg", LegalValues = new object[] { "bmp", "gif", "jpg", })]
    public string   ext = "bmp";

    [clp.NoUsage]
    public string   autoexec = @"c:\autoexec.bat";

    [clp.ValueUsage("Location of autoexec.bat file.", Name = "autoexec")]
    public string   Autoexec
    {
        get { return autoexec; }
        set 
        {
            if( !(new FileInfo(value)).Exists )
            {
                throw new UsageException("autoexec", value + " not found");
            }

            autoexec = value;
        }
    }

    [clp.ValueUsage("Include directories.", Name = "I", IgnoreCase = false)]
    public ArrayList includes = new ArrayList();

    [clp.ValueUsage("circumference/diameter", MatchPosition = true, Optional = false)]
    public double   pi = 3.0;

    [clp.ValueUsage("File names.", Name = "file", MatchPosition = true)]
    public ArrayList files = new ArrayList();
}

class App
{
    [STAThread]
    static void Main(string[] args)
    {
        // Parse the command line and show help or version or error
        TestCommandLine     cl = new TestCommandLine();
        if( !cl.ParseAndContinue(args) ) return;

        // Harvest values
        Console.WriteLine("Harvested values:");
        Console.WriteLine("debug: " + cl.debug);
        Console.WriteLine("regserver: " + cl.regserver);
        Console.WriteLine("minX: " + cl.minX);
        Console.WriteLine("intensity: " + cl.intensity);
        Console.WriteLine("foo: " + cl.foo);
        Console.WriteLine("pi: " + cl.pi);
        Console.WriteLine("ext: " + cl.ext);
        Console.WriteLine("autoexec: " + cl.autoexec);

        Console.WriteLine("includes:");
        foreach( string incl in cl.includes )
        {
            DirectoryInfo   info = new DirectoryInfo(incl);
            Console.WriteLine("\t'{0}' [{1}] ({2})", incl, info.FullName, info.Exists ? "exists" : "doesn't exist");
        }

        Console.WriteLine("files:");
        foreach( string file in cl.files )
        {
            FileInfo    info = new FileInfo(file);
            Console.WriteLine("\t'{0}' [{1}] ({2})", file, info.FullName, info.Exists ? "exists" : "doesn't exist");
        }
    }
}
