// CompletionCombo.cs: Contributed by Chris Sells [csells@sellsbrothers.com]
// Combo Box that does that neat auto-completion trick
#region Copyright  2002-2003 The Genghis Group
/*
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation is required, as shown here:
 * 
 * Portions Copyright  2002-2003 The Genghis Group (http://www.genghisgroup.com/).
 * 
 * 2. No substantial portion of the source code of this library may be redistributed
 * without the express written permission of the copyright holders, where
 * "substantial" is defined as enough code to be recognizably from this library. 
*/
#endregion

using System;
using System.Diagnostics;   // Trace and Debug.Assert
using System.Windows.Forms;
using System.ComponentModel;

namespace Genghis.Windows.Forms
{
    /// <summary>
    /// Combo Box that does that neat auto-completion trick
    /// </summary>
    public class CompletionCombo : ComboBox
    {
        public System.ComponentModel.CancelEventHandler NotInList;
 
 
        [Category("Behavior")]
        public bool LimitToList
        {
            get { return _limitToList; }
            set { _limitToList = value; }
        }
         
        protected virtual void OnNotInList(System.ComponentModel.CancelEventArgs e)
        {
            if (NotInList != null)
                NotInList.Invoke(this, e);
        }
         
        protected override void OnValidating(System.ComponentModel.CancelEventArgs e)
        {
            if (_limitToList)
            {
                int pos = FindStringExact(Text);
                if (pos == -1)
                    OnNotInList(e);
                else
                    this.SelectedIndex = pos;
            }
         
            base.OnValidating(e);
        }


        protected override void OnKeyDown(KeyEventArgs args)
        {
            // Need to check for backspace/delete. These will modify the text in
            // the edit box, causing the auto complete to just add back the text
            // the user has just tried to delete. 
            _autoComplete = args.KeyCode != Keys.Delete && args.KeyCode != Keys.Back;
            base.OnKeyDown(args);
        }

        protected override void OnTextChanged(EventArgs args)
        {
            if( _autoComplete )
            {
                // Find the current string in the list of items in the list
                string  textEntered = Text;
                int index = FindString(textEntered);
                if( index >= 0 )
                {
                    // If the entered string is found in the list,
                    // select it in the list and set the edit selection
                    // range to everything past the entered text

                    // NOTE: Changing the selected index will cause an OnTextChanged event,
                    // so don't auto complete while changing the selected index.

                    _autoComplete = false;
                    SelectedIndex = index;
                    _autoComplete = true;

                    // This selection range will be trimmed, so it's OK to be
                    // longer than needed. Using Text.Length gives us the
                    // guaranteed maximum
                    Select(textEntered.Length, Text.Length);
                }
            }

            base.OnTextChanged(args);
        }

        private bool  _autoComplete = true;
        private bool  _limitToList = true;
    }
}
