// CoolMenu.cs: Contributed by Chris Burrows [chris_j_burrows@hotmail.com]
// Owner-draw menu with icons (in the style of VS.NET or Windows Movie Maker).
#region Copyright 2002 The Genghis Group
///*  Copyright 2002 The Genghis Group
//    This software is provided 'as-is', without any express or implied warranty. 
//    In no event will the authors be held liable for any damages arising from the use of this software.
//    Permission is granted to anyone to use this software for any purpose, including commercial applications, 
//    subject to the following restrictions:
//    
//    The origin of this software must not be misrepresented; you must not claim that you wrote 
//    the original software. If you use this software in a product, an acknowledgment in the product 
//    documentation is required, as shown here:
//
//        Portions copyright 2002 The Genghis Group (http://www.genghisgroup.com/).
//  
//    No substantial portion of the source code of this library may be redistributed without the 
//    express written permission of the copyright holders, where "substantial" is defined as enough 
//    code to be recognizably from this library. 
//*/
#endregion Copyright

using System;
using System.Windows.Forms;
using System.Xml;
using System.Drawing;
using System.Drawing.Text;
using Genghis;

namespace Genghis.Windows.Forms
{
    #region MenuTypeEnum
    /// The Type of menu that will be displayed
    /// </summary>
    public enum MenuType
    {
        VisualStudio,
        WindowsMovieMaker
    }
    #endregion

    #region CoolMainMenu Class
    /// <summary>
    /// A Cool Menu is a collection of menu item that make up 
    /// the menus for an application
    /// </summary>
    /// 

    public class CoolMainMenu : CommandMainMenu
    {

        //List of Todo's
        //TODO:  Implement Left to right and right to left.


        private MenuType menuType ;

        /// <summary>
        ///  Default Constructor
        /// </summary>
        public CoolMainMenu():base()
        {
            this.MenuType = MenuType.VisualStudio;
        }

       


        /// <summary>
        /// Create a cool main menu from an existing menu item
        /// </summary>
        /// <param name="menu">The main menu from which to create the menu</param>
        public CoolMainMenu(MainMenu menu):this()
        {

            //If the menu if null don't process it any further
            if (menu != null)
            {
                this.CloneMenu(menu);
                this.MenuItems.Clear();
                foreach (MenuItem menuItem in menu.MenuItems)
                {
                    this.MenuItems.Add(CoolMenuItem.Load(menuItem));
                }
            }
        }

        /// <summary>
        /// What type of menu is the main menu
        /// </summary>
        public MenuType MenuType
        {
            get
            {
                return this.menuType;
            }
            set
            {
                this.menuType = value;
                //TODO: Figure out a way to inform the children that the menu type has changed
                
            }
        }
    }
        
    
    #endregion CoolMainMenu
    
    #region CoolMenuItem
    /// <summary>
    /// Summary description for CoolMenuItem.
    /// </summary>
    public class CoolMenuItem : CommandMenuItem
    {

        //List of TODO's
        //TODO:  Handle RadioCheck mark.
        
        //A const that indicates how much the buffer at the start and end of the text should be
        private const int TEXT_BUFFER = 5;
        private const int IMAGE_BUFFER = 4;
        private const int SEPERATOR_HEIGHT = 8;
        
        
        private Image image;

       #region Private Variables and Private Properties
      
        private MenuType MenuType
        {
            get
            {
                //TODO: Handle Context Menu
                CoolMainMenu menu =(CoolMainMenu)this.GetMainMenu();
                if (menu != null)
                {
                    return menu.MenuType;
                }
                else
                {
                    //Defaults to Visual Studio
                    return MenuType.VisualStudio;
                }

            }
        }

        


        /// <summary>
        /// This Checks to see if the given menu is a Top Level Menu
        /// </summary>
        /// <returns>True if the menu is a top level menu, false otherwise</returns>
        private bool IsTopLevelMenu
        {
            get
            {
                //TODO: Confirm that this OK for Context Menus. 
                if (this.Parent == null)
                {
                    return false;
                }
                
                return this.Parent == this.Parent.GetMainMenu();
            }
        }

        /// <summary>
        /// Gets the Font to use with this menu
        /// </summary>
        private Font Font
        {
            get
            {
                return SystemInformation.MenuFont;
            }
        }
        #endregion
       #region Cool Menu Item Constructors


        private CoolMenuItem():base()
        {
            //Always draw this menu ourselves
            this.OwnerDraw = true;

        }
        
        /// <summary>
        /// Constructor that construsts a menu item out the given command
        /// </summary>
        /// <param name="command"></param>
        public CoolMenuItem(Command command):this()
        {
            this.Command= command;
        }
		
        
        /// <summary>
        /// Constructor that takes a menu item
        /// and turns it into a cool menu
        /// </summary>
        /// <param name="menuItem">The menu item to convert to a cool menu</param>
        public CoolMenuItem(MenuItem menuItem):this()
        {
            
            //TODO: Building the Command Handler and hooking in with Pauls Command
            this.CloneMenu(menuItem);
            //When you clone the menu owner draw is changed. Set it back to true
            this.OwnerDraw = true;
            //Bit of an ugly way to do it but it works. We want a shallow copy of the menu not a deep copy.
            //Therefore, remove any of the menu items. 
            this.MenuItems.Clear();
        }
        
        /// <summary>
        /// Simple constructor to construct a simple menu with just text in it
        /// </summary>
        /// <param name="text">The text to appear on the menu</param>
        public CoolMenuItem(string text):this()
        {
           this.Text = text;
        }


        /// <summary>
        /// Constructor to construct a menu item with text and an image associated with it
        /// </summary>
        /// <param name="text">The text to give to the menu item</param>
        /// <param name="image">The image to associate with the menu</param>
        public CoolMenuItem(string text,Bitmap image):this(text)
        {
            this.Image = image;
        }

        #endregion

       #region Static Load Methods
        
        /// <summary>
        /// Creates a menu item and loads any children associated with it from an XML File
        /// </summary>
        /// <param name="menus">The XML node to process</param>
        /// <returns>A new CoolMenuItem and it's assoicated children</returns>
        /// 
      
        /// <summary>
        /// Create a new Cool Menu item from a given Menu item.  Used to
        /// take an older style of menu item and create a new one from it.
        /// </summary>
        /// <param name="menuItem">The menu item to create the Cool Menu From</param>
        /// <returns>A new cool menu that corresponds to the given menu item</returns>
        public static CoolMenuItem Load(MenuItem menuItem)
        {
            //TODO:  Check that the event Handlers still work
            CoolMenuItem newMenu = new CoolMenuItem(menuItem);
            foreach (MenuItem newMenuItem in menuItem.MenuItems)
            {
                newMenu.MenuItems.Add(CoolMenuItem.Load(newMenuItem));
            }
            return newMenu;
        }
        #endregion

        /// <summary>
        /// Sets the Image of the Menu Item
        /// </summary>
        public Image Image
        {
            get
            {
                return this.image;
            }
            set
            {
                this.image = value;
            }
        }

        #region Methods required to Draw the menu

        /// <summary>
        /// Overriden Draw item event.  This needs to be overriden so that we can 
        /// draw the menu ourselves.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDrawItem(System.Windows.Forms.DrawItemEventArgs e)
        {
            //Double Buffering techique
            int width = e.Bounds.Width;
            int height = e.Bounds.Height;

            Bitmap menuImage = new Bitmap(width,height);

            // Adjust bounds origin to (0, 0).
            Rectangle bounds = e.Bounds;
            bounds.Location = Point.Empty;

            Graphics g = Graphics.FromImage(menuImage);
    

            //Draw the Background of the Menu
            this.DrawMenuBackground(this,bounds,g,e.State);

            //Draw the image if one is associate with the command\
            if (this.Checked)
            {
                this.DrawCheckMark(bounds,g, e.State);
            }
          


            //Only draw the image if the menu Item is not checked
            //and if it isn't a seperator
            if (!this.Checked && !this.IsSeperator)
            {
                this.DrawImage(this.Image, bounds,g, e.State);
            }

            //If this is a seperator we need to draw the seperator instead of the text
            if (this.IsSeperator)
            {
                this.DrawSeparator(g,bounds);
            }
            else
            {
                this.DrawMenuText(bounds, g,e.State);
            }

            //Draw the image
            
            e.Graphics.DrawImage(menuImage,e.Bounds); 
            g.Dispose();
            menuImage.Dispose();
            e.Graphics.Dispose();    
        }

        /// <summary>
        /// Overriden OnMeasureItemEvent.  This needs to be ovveridden so that we
        /// can respond to the Measure item event and calculate the menu size ourself.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMeasureItem(System.Windows.Forms.MeasureItemEventArgs e)
        {

            base.OnMeasureItem(e);
          
                                                        
            Size imageSize = this.ImageSize;
            
            Size textSize = this.GetTextSize(e.Graphics);
            
            //Width of the menu will be width of the image, + width of the text+start text buffer + end text buffer
            if (this.IsTopLevelMenu)
                e.ItemWidth = textSize.Width; 
            else
                e.ItemWidth = this.ImageSize.Width + textSize.Width + 2*TEXT_BUFFER; 
            

            
            //Check to see if the height of the image is higher then the 
            //Height of the menu.  If so then we need to use the height of the image
            //instead.
            if (SystemInformation.MenuHeight > imageSize.Height)
            {
                e.ItemHeight =  SystemInformation.MenuHeight;
            }
            else
            {
                e.ItemHeight = imageSize.Height ;
            }

            //Last Check to see if this is a seperator. If it is then we need to change the height
            if (this.IsSeperator)
            {
                e.ItemHeight = SEPERATOR_HEIGHT;
            }
        }


        /// <summary>
        /// Gets the string formatter to use to format the menu text
        /// </summary>
        /// <param name="itemState">The state of the menu state</param>
        /// <returns>
        /// a string formatter that represents the formatter that should be used for the menu text
        /// </returns>
        private StringFormat GetMenuTextFormatter(DrawItemState itemState)
        {
            StringFormat menuFormat = new StringFormat();

            //Check to see if Accelerators should be shown, or if the text 
            //has a & character in it.

            if ((itemState & DrawItemState.NoAccelerator) != 0)
            {
                menuFormat.HotkeyPrefix = HotkeyPrefix.Hide;
            }
            else
            {
                menuFormat.HotkeyPrefix = HotkeyPrefix.Show;
            }
            return menuFormat;
        }

        /// <summary>
        /// Encapsulates getting the menu text brush.  Hides the complexity of what
        /// brush to use when drawing menu text
        /// </summary>
        private Brush GetTextBrush(DrawItemState itemstate)
        {
           
            //Get the correct brush to use
            if (this.Enabled)
            {
                if (this.MenuType == MenuType.WindowsMovieMaker && this.IsMenuSelected(itemstate))
                {
                    return SystemBrushes.HighlightText;
                }
                else
                {
                    return SystemBrushes.ControlText;
                }
            }
            else
            {
                return SystemBrushes.InactiveBorder;
            }
        }

        /// <summary>
        /// This function is used to determine the size of the text.  
        /// </summary>
        /// <param name="g">A Graphics object so we can measure the size of the text</param>
        /// <returns>A Size object that corresponds to the size of the text</returns>
        /// <note type="note">This doesn't take into account the buffer region that will be required around the text.</note>
        private Size GetTextSize(Graphics g)
        {
            string text = this.Text.Replace("&","") + this.ShortcutText;
            SizeF textSize = g.MeasureString(text, this.Font);
            return new Size(Convert.ToInt32(textSize.Width),Convert.ToInt32(textSize.Height));
        }


        

        /// <summary>
        /// Gets the Size of the Image
        /// </summary>
        private Size ImageSize
        {
            //TODO: Figure out a way to cache this as it is called numerous times.
            get
            {

                if (this.IsTopLevelMenu)
                {
                    return new Size(0,0);
                }
                else
                {
                    Size imageSize = SystemInformation.SmallIconSize;
                    //If the image hasn't been defined then we need to get the default image size
                    if (imageSize.IsEmpty || imageSize.Width == 0 || imageSize.Height == 0)
                    {
                        //CHECK: What should the default size be
                        return new Size(16 + IMAGE_BUFFER*2,16 +IMAGE_BUFFER);
                    }
                    else
                    {
                        return new Size(imageSize.Width +IMAGE_BUFFER*2, imageSize.Height+IMAGE_BUFFER);
                    }
                }
            }

        }

        #region Helper functions to draw the menu and it's background
        /// <summary>
        /// This checks to see if the menu has been selected
        /// </summary>
        /// <param name="menuState">The state of the menu</param>
        /// <returns>Returns true if the menu has been selected, otherwise false</returns>
        private bool IsMenuSelected(DrawItemState menuState)
        {
            return (((menuState & DrawItemState.Selected) != 0) || (menuState & DrawItemState.HotLight) != 0);
        }

      
     
        /// <summary>
        /// Gets the shortcut text assoicted with this item
        /// </summary>
        private string ShortcutText
        {

            //TODO: Decide if this needs to cached or not
            get
            {
                string shortCutText = "";
                //Only draw the shortcut if required and it is not a top menu
                if (this.Shortcut != Shortcut.None && this.ShowShortcut && !this.IsTopLevelMenu)
                {
                    //Change the Short Cut to and int so we can do comparisions
                    int  shortcut = (int)this.Shortcut;
                    
                    //Grab the character section of the shortcut
                    int  character   = shortcut & 0xFF;

                    //Does the shortkey include a control key
                    if (((int)Keys.Control & shortcut) > 0)
                        shortCutText += "Ctrl+";

                    //Does the shortkey include a Shift key
                    if (((int)Keys.Shift & shortcut) > 0)
                        shortCutText += "Shift+";

                    //Does the shortkey include a Alt key
                    if (((int)Keys.Alt & shortcut) > 0)
                        shortCutText += "Alt+";
				
                    //Compare the character
                    if (character >= (int)Shortcut.F1 && character <= (int)Shortcut.F12)
                        shortCutText += "F" + (character - (int)Shortcut.F1 + 1);
                    else 
                    {
                        if ( Shortcut == Shortcut.Del) 
                        {
                            shortCutText += "Del";
                        }
                        else 
                        {
                            shortCutText += (char)character;
                        }
                    }
                 
                }
                return shortCutText;
            }
        }
        
        /// <summary>
        /// This draws the background of the menu item.
        /// </summary>
        /// <param name="thisMenu">The menu item to draw the background of </param>
        /// <param name="menuBounds">The rectangle that indicates the bounds of the menu</param>
        /// <param name="g">The graphics device to draw the item on</param>
        /// <param name="itemState"></param>
        private void DrawMenuBackground(MenuItem thisMenu,Rectangle menuBounds, Graphics g, DrawItemState itemState)
        {
            
            //If the menu has been selected then we don't care about anything 
            //else

            Brush backgroundBrush = this.GetMenuBackgroundBrush(thisMenu,itemState);

            
            
            //If Menu item is selected or hotlighted
            if (this.Enabled && this.IsMenuSelected(itemState))
            {

                g.FillRectangle(backgroundBrush , menuBounds);   
                //TODO: Change to the correct colour
                if (this.MenuType == MenuType.WindowsMovieMaker)
                {
                    //ControlPaint.DrawFocusRectangle(g,menuBounds);
                }
                else
                {
                    float width = menuBounds.Width - 0.25f;
                    float height = menuBounds.Height - 0.25f;
                    //g.DrawRectangle(SystemPens.Highlight,menuBounds.X,menuBounds.Y, width, height );
                    ControlPaint.DrawBorder(g,menuBounds,SystemColors.Highlight,ButtonBorderStyle.Solid);
                }

            }
            else
            {
                g.FillRectangle(backgroundBrush , menuBounds); 
                if (this.MenuType == MenuType.VisualStudio)
                {
                    //Draw the Background of the Menu
                    this.DrawImageBackground(g,menuBounds);
                }
            }

           
        }

        /// <summary>
        /// This function draws the background for the image.
        /// </summary>
        /// <param name="g">The Graphics device to draw the background for the image on</param>
        /// <param name="menuBounds">The Rectangle that indicates the bounds of the menu</param>
        private void DrawImageBackground(Graphics g, Rectangle menuBounds)
        {
            //TODO Need to change this function so it takes the image width as well
            Brush backgroundBrush = this.GetImageBackgroundBrush();
            Rectangle imageRectangle = new Rectangle(menuBounds.X,menuBounds.Y,this.ImageSize.Width,menuBounds.Height);
            g.FillRectangle(backgroundBrush,imageRectangle);

        }


        /// <summary>
        /// Draws the menu text
        /// </summary>
        /// <param name="g"></param>
        /// <param name="itemState"></param>
        /// <param name="menuBounds"></param>
        private void DrawMenuText(Rectangle menuBounds,Graphics g, DrawItemState itemState)
        {
           
            StringFormat menuFormat = this.GetMenuTextFormatter(itemState);
            Size textSize = this.GetTextSize(g);
            //Need to get the start location for the text.
            //Try and centralise it.
            float textYLocation = menuBounds.Y + (menuBounds.Height - textSize.Height)/2;
            float textXLocation;
            //If this is a top level menu then we should try and centre the text
            if (this.IsTopLevelMenu )
            {
                //TODO:Refactor the way that the X, and Y locations are calulated. This should be in a different function
                textXLocation = menuBounds.X + (menuBounds.Width-textSize.Width)/2;
            }
            else
            {
                 textXLocation = menuBounds.X + this.ImageSize.Width + TEXT_BUFFER;
            }

            g.DrawString(this.Text, this.Font, this.GetTextBrush(itemState),textXLocation ,textYLocation,menuFormat);
            
            //Draw the short cut text
            if (this.ShowShortcut)
                this.DrawShortCutText(menuBounds,g,itemState);

            menuFormat.Dispose();
        }


        /// <summary>
        /// Draw the short cut text if required
        /// </summary>
        /// <param name="menuBounds"></param>
        /// <param name="g"></param>
        /// <param name="itemState"></param>
        private void DrawShortCutText(Rectangle menuBounds,Graphics g, DrawItemState itemState)
        {
            StringFormat menuFormat = this.GetMenuTextFormatter(itemState);
            
            //Shortcut text should be right aligned
            menuFormat.FormatFlags |= StringFormatFlags.DirectionRightToLeft;
            //Need to get the start location for the text.
            //Try and centralise it.
            float textYLocation = menuBounds.Y + (menuBounds.Height - this.GetTextSize(g).Height)/2;
            float textXLocation = menuBounds.Width - TEXT_BUFFER;
            g.DrawString(this.ShortcutText, this.Font, this.GetTextBrush(itemState),textXLocation ,textYLocation,menuFormat);
            menuFormat.Dispose();
        }
        
        /// <summary>
        /// Draw the seperator mark.
        /// </summary>
        /// <param name="g"></param>
        /// <param name="bounds"></param>
        private void DrawSeparator(Graphics g, Rectangle menuBounds)
        {
            float yLocation,xLocation;

            //If the menu type is movie maker then the seperator is displayed all the way along
            if (this.MenuType == MenuType.WindowsMovieMaker)
            {    
                yLocation = menuBounds.Y + (menuBounds.Height)/2;
                xLocation = menuBounds.X;//  + TEXT_BUFFER;
                
            }
            else
            {
                yLocation = menuBounds.Y + (menuBounds.Height)/2;
                xLocation= menuBounds.X + this.ImageSize.Width + TEXT_BUFFER;
            }
            //CHECK: Check that this is right type of Pen to uses
            g.DrawLine(new Pen(SystemBrushes.Control), xLocation, yLocation, menuBounds.Width,yLocation);
        }

        /// <summary>
        /// Draw a Check Mark against items that are checked
        /// </summary>
        /// <param name="menuBounds"></param>
        /// <param name="g"></param>
        /// <param name="itemState"></param>
        private void DrawCheckMark(Rectangle menuBounds, Graphics g, DrawItemState itemState)
        {
            if (this.Checked)
            {
                int checkXLocation =  menuBounds.X + IMAGE_BUFFER/2;
                int checkYLocation =  menuBounds.Y + (menuBounds.Height - this.ImageSize.Height)/2 + IMAGE_BUFFER /2; 
                //TODO: Figure out why the check mark background is being painted in a different color.
                //TODO: Figure out what the check mark isn't centred.
                ControlPaint.DrawMenuGlyph(g, checkXLocation,  checkYLocation,this.ImageSize.Width -IMAGE_BUFFER, this.ImageSize.Height -IMAGE_BUFFER, MenuGlyph.Checkmark);
                ControlPaint.DrawBorder3D(g,checkXLocation,  checkYLocation,this.ImageSize.Width -IMAGE_BUFFER, this.ImageSize.Height -IMAGE_BUFFER,
                                          Border3DStyle.Flat, Border3DSide.Bottom | Border3DSide.Top |
                                          Border3DSide.Left | Border3DSide.Right);
                
            }
        }

        /// <summary>
        /// Draws the Image
        /// </summary>
        /// <param name="image"></param>
        /// <param name="menuBounds"></param>
        /// <param name="g"></param>
        /// <param name="itemState"></param>
        private void DrawImage(Image image, Rectangle menuBounds, Graphics g, DrawItemState itemState)
        {

            
            if (image != null && !this.IsTopLevelMenu)
            {
                
                //Centre the image
                int imageYLocation = menuBounds.Y + (menuBounds.Height - this.ImageSize.Height)/2 + IMAGE_BUFFER/2; 
                
                //Handle if the menu is disabled.
                if (this.Enabled)
                {
                    //If the menu item is selected then raise the image
                    if (this.IsMenuSelected(itemState) && this.MenuType != MenuType.WindowsMovieMaker)
                    {
                       //CHECK: That Black is a good color to use
                        ControlPaint.DrawImageDisabled(g, image, menuBounds.X +3,imageYLocation,Color.Black);
                        g.DrawImage(image,menuBounds.X + 2,imageYLocation -1);  
                    }
                    else
                    {
                        
                        g.DrawImage(image,menuBounds.X + 2,imageYLocation);  
                    }
                }
                else
                {
                     ControlPaint.DrawImageDisabled(g, image, menuBounds.X +2,imageYLocation,Color.Black);
                }
                
            }
        }

        
        /// <summary>
        /// This function gets the brush that will be used to draw the background
        /// </summary>
        /// <param name="thisMenu"></param>
        /// <param name="itemState"></param>
        /// <returns></returns>
        private Brush GetMenuBackgroundBrush(MenuItem thisMenu, DrawItemState itemState)
        {

            //If the item is selected it doesn't matter if the
            //item is a top menu item or not as we use the same brush to
            //paint with
            

            if (!this.Enabled)
            {
                return  new SolidBrush(ColorHelper.Background);
            }
          
            if (this.IsMenuSelected(itemState ))
            {
                if (this.MenuType == MenuType.WindowsMovieMaker)
                {
                    return SystemBrushes.Highlight;
                }
                else
                {
                    return new SolidBrush(ColorHelper.Selection);
                }
            }

            if (this.IsTopLevelMenu)
            {
                return SystemBrushes.Control;
            }
            
            
            return  new SolidBrush(ColorHelper.Background);
            
        }

        

        /// <summary>
        /// Gets the Brush to use when drawing the background image
        /// </summary>
        /// <returns></returns>
        private Brush GetImageBackgroundBrush()
        {
            return SystemBrushes.Control;
        }
       
        #endregion

        protected override void OnCommandChanged(EventArgs e) 
        {
            base.OnCommandChanged(e);
            this.Image = base.Command.Image;
        }

     
        /// <summary>
        /// Indicates whether or not this menu item is a seperator
        /// </summary>
        public bool IsSeperator
        {
            get
            {
                return (this.Text == "-") ;
            }

        }

        
        #endregion

        #region ColorHelper Class
        /// <summary>
        /// This class is used to provide the colours for the menus.  
        /// Thanks to Carlos Perez for this code.
        /// </summary>
        class ColorHelper
        {
            private static Color CalculateColor(Color front, Color back, int alpha)
            {
			
                // Use alpha blending to brigthen the colors but don't use it
                // directly. Instead derive an opaque color that we can use.
                // -- if we use a color with alpha blending directly we won't be able 
                // to paint over whatever color was in the background and there
                // would be shadows of that color showing through
                Color frontColor = Color.FromArgb(255, front);
                Color backColor = Color.FromArgb(255, back);
									
                float frontRed = frontColor.R;
                float frontGreen = frontColor.G;
                float frontBlue = frontColor.B;
                float backRed = backColor.R;
                float backGreen = backColor.G;
                float backBlue = backColor.B;
			
                float fRed = frontRed*alpha/255 + backRed*((float)(255-alpha)/255);
                byte newRed = (byte)fRed;
                float fGreen = frontGreen*alpha/255 + backGreen*((float)(255-alpha)/255);
                byte newGreen = (byte)fGreen;
                float fBlue = frontBlue*alpha/255 + backBlue*((float)(255-alpha)/255);
                byte newBlue = (byte)fBlue;

                return  Color.FromArgb(255, newRed, newGreen, newBlue);
            }

            public static Color Selection
            {
                get
                {
                    return CalculateColor(SystemColors.Highlight,SystemColors.Window, 70);
                }
            }

            public static Color Background
            {
                get
                {
                    return CalculateColor(SystemColors.Window,SystemColors.Control, 200);
                }
            }

            public static Color Control
            {
                get
                {
                    return CalculateColor(SystemColors.Control,ColorHelper.Background, 195);
                }

            }
        } 

        #endregion
    }
    #endregion CoolMenuItem
}
