// textBoxData.cs: TextBoxData class

using System;
using System.IO;
using System.Runtime.Serialization;
using System.Windows.Forms;

namespace FileDocumentTest
{
    // Document adapter around the TextBox type to handle serialization,
    // but to avoid replicating data into the document whenever the
    // TextBox data changes.
    [Serializable]
    internal class TextBoxData : ISerializable
    {
        public TextBoxData(TextBox textBox)
        {
            _textBox = textBox;
        }

        #region Implementation of ISerializable
        public TextBoxData(SerializationInfo info, StreamingContext context)
        {
            Text = (string)info.GetString("_text");
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("_text", Text);
        }
        #endregion

        // Manage the cached text or the textbox's text
        public string Text
        {
            get
            {
                if( _textBox != null )
                {
                    return _textBox.Text;
                }
                else
                {
                    return _text;
                }
            }

            set
            {
                if( _textBox != null )
                {
                    _textBox.Text = value;
                }
                else
                {
                    _text = value;
                }
            }
        }

        public TextBox TextBox
        {
            get
            {
                return _textBox;
            }

            set
            {
                // Cache existing textbox's text
                if( _textBox != null )
                {
                    _text = _textBox.Text;
                }

                // Move existing text into new textbox
                _textBox = value;

                if( _textBox != null )
                {
                    _textBox.Text = _text;
                    _textBox.Select(0, 0);
                    _text = null;
                }
            }
        }

        TextBox _textBox = null;
        string  _text = null;
    }
}
