using System.Collections;
using System.Windows.Forms;
using Genghis;

namespace Genghis.DocView {

	public class View : UserControl, ICommandTarget {
		Document document;

		Hashtable executeHandlers;
		Hashtable updateHandlers;

		public View() {
			executeHandlers = Command.GetExecuteHandlers(this);
			updateHandlers = Command.GetUpdateHandlers(this);
		}

		public View(Document document) : this() {
			this.document = document;
			if (document != null)
				document.Views.Add(this);
		}

		// Used to hook up command event handlers.
		public Hashtable ExecuteHandlers {
			get { return executeHandlers; }
		}
		public Hashtable UpdateHandlers {
			get { return updateHandlers; }
		}

		public Document Document {
			get { return document; }
		}
		
		// Called by ViewContainer when the window is closed.
		public void Close() {
			document.Views.Remove(this);
		}
		
		// Called when this view is added to a document.
		internal void SetDocument(Document document) {
			this.document = document;
			if (document != null)
				OnDocumentChanged();
		}
		
		protected internal virtual void OnDocumentChanged() {
			
		}
	}

}