// FolderNameDialog.cs: Contributed by Chris Sells [csells@sellsbrothers.com]
// Exposing the design-time SHBrowseForFolder wrapper
// Inspired by code from Walt Ritscher [walt_ritscher@INTELLI-TRAK.COM]

#region Copyright  2002-2003 The Genghis Group
/*
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation is required, as shown here:
 * 
 * Portions Copyright  2002-2003 The Genghis Group (http://www.genghisgroup.com/).
 * 
 * 2. No substantial portion of the source code of this library may be redistributed
 * without the express written permission of the copyright holders, where
 * "substantial" is defined as enough code to be recognizably from this library. 
*/
#endregion
#region History
// 7/15/02: Deepak Shenoy [shenoy@agnisoft.com] added a bunch of extra flags.
#endregion
#region Limitations
// -Ideally there should be an "Options" property containing
//  sub-properties such as BrowseForComputer etc.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace Genghis.Windows.Forms
{

  /// <summary>
    /// Summary description for ImageCombo.
    /// </summary>
  [
  DesignTimeVisible(true),
  ToolboxItem(true)
  ]
  public class FolderNameDialog : FolderNameEditor, IComponent
  {
    public enum FolderNameStyles
    {
      BrowseForComputer       = 0x1000,
      BrowseForEverything     = 0x4000,
      BrowseForPrinter        = 0x2000,
      DontTranslateTarget     = 0x0400,
      NewUI                   = 0x0040,
      NoNewFolderButton       = 0x0200,
      RestrictToDomain        = 0x0002,
      RestrictToFilesystem    = 0x0001,
      RestrictToSubfolders    = 0x0008,
      ShowHint                = 0x0100, /// <summary>Add a UA hint to the dialog, in place of the edit box. May not be combined with <c>ShowTextBox</c></summary>
      ShowShareable           = 0x8000, /// <summary> requires <c>NewUI</c> </summary>
      ShowStatusText          = 0x0004,
      ShowTextBox             = 0x0010,
      ValidResultsOnly        = 0x0020,
    };  

    public enum FolderNameFolder
    {
      Desktop                 = 0x00,
      Favorites               = 0x06,
      MyComputer              = 0x11,
      MyDocuments             = 0x05,
      MyPictures              = 0x27,
      NetAndDialUpConnections = 0x31,
      NetworkNeighborhood     = 0x12,
      Printers                = 0x04,
      Recent                  = 0x08,
      SendTo                  = 0x09,
      StartMenu               = 0x0B,
      Templates               = 0x15,
    };


    public FolderNameDialog()
    {
      Description = "Choose a folder";
      FBrowseForComputer = false;
      FBrowseForEverything = false;
      FBrowseForPrinter = false;
      FDontTranslateTarget = false;
      FNewUI = true;
      FNoNewFolderButton = false;
      FRestrictToDomain = false;
      FRestrictToFilesystem = false;
      FRestrictToSubfolders  = false;
      FShowHint = false;
      FShowShareable = false;
      FShowTextBox = false;
      FValidResultsOnly = false;
    }

    public DialogResult ShowDialog()
    {
      dlg.Style = (FolderNameEditor.FolderBrowserStyles)ConvertStyle();
      return dlg.ShowDialog();
    }

    public DialogResult ShowDialog(IWin32Window owner)
    {
      dlg.Style = (FolderNameEditor.FolderBrowserStyles)ConvertStyle();
      return dlg.ShowDialog(owner);
    }

    public string Description
    {
      get { return dlg.Description; }
      set { dlg.Description = value; }
    }

    public string DirectoryPath
    {
      get { return dlg.DirectoryPath; }
    }
        
      [Category("Misc")]
      public bool BrowseForComputer
      {
        get { return FBrowseForComputer; }
        set { FBrowseForComputer = BrowseForComputer;}
      }
      
        [Category("Misc")]
        public bool BrowseForEverything
        { 
          get { return FBrowseForEverything; }
          set { FBrowseForEverything = value; }
        }
        [Category("Misc")]
        public bool BrowseForPrinter
        { 
          get { return FBrowseForPrinter; }
          set { FBrowseForPrinter = value; }
        }
        [Category("Misc")]
        public bool DontTranslateTarget
        { 
          get { return FDontTranslateTarget; }
          set { FDontTranslateTarget = value; }
        }
        [Category("Misc")]
        public bool NewUI
        { 
          get { return FNewUI; }
          set { FNewUI = value; }
        }
        [Category("Misc")]
        public bool NoNewFolderButton
        { 
          get { return FNoNewFolderButton; }
          set { FNoNewFolderButton = value; }
        }
        [Category("Misc")]
        public bool RestrictToDomain
        { 
          get { return FRestrictToDomain; }
          set { FRestrictToDomain = value; }
        }
        [Category("Misc")]
        public bool RestrictToFilesystem
        { 
          get { return FRestrictToFilesystem; }
          set { FRestrictToFilesystem = value; }
        }
        [Category("Misc")]
        public bool RestrictToSubfolders
        { 
          get { return FRestrictToSubfolders; }
          set { FRestrictToSubfolders = value; }
        }
        [Category("Misc")]
        public bool ShowHint
        { 
          get { return FShowHint; }
          set { FShowHint = ShowHint; }
        }
        [Category("Misc")]
        public bool ShowShareable
        { 
          get { return FShowShareable; }
          set { FShowShareable = value; }
        }
        [Category("Misc")]
        public bool ShowStatusText
        { 
          get { return FShowStatusText; }
          set { FShowStatusText = value; }
        }
        [Category("Misc")]
        public bool ShowTextBox
        { 
          get { return FShowTextBox; }
          set { FShowTextBox = value; }
        }
        [Category("Misc")]
        public bool ValidResultsOnly
        { 
          get { return FValidResultsOnly; }
          set { FValidResultsOnly = value; }
        }

    
        int ConvertStyle()
        {
          int WinStyle=0;
          if (BrowseForComputer) 
            WinStyle = WinStyle + (int) FolderNameStyles.BrowseForComputer;
          if (BrowseForEverything) 
            WinStyle = WinStyle + (int) FolderNameStyles.BrowseForEverything;
          if (BrowseForPrinter) 
            WinStyle = WinStyle + (int) FolderNameStyles.BrowseForPrinter;
          if (DontTranslateTarget) 
            WinStyle = WinStyle + (int) FolderNameStyles.DontTranslateTarget;
          if (NewUI) 
            WinStyle = WinStyle + (int) FolderNameStyles.NewUI;
          if (NoNewFolderButton) 
            WinStyle = WinStyle + (int) FolderNameStyles.NoNewFolderButton;
          if (RestrictToDomain) 
            WinStyle = WinStyle + (int) FolderNameStyles.RestrictToDomain;
          if (RestrictToFilesystem) 
            WinStyle = WinStyle + (int) FolderNameStyles.RestrictToFilesystem;
          if (RestrictToSubfolders) 
            WinStyle = WinStyle + (int) FolderNameStyles.RestrictToSubfolders;
          if (ShowHint) 
            WinStyle = WinStyle + (int) FolderNameStyles.ShowHint;
          if (ShowShareable) 
            WinStyle = WinStyle + (int) FolderNameStyles.ShowShareable;
          if (ShowStatusText) 
            WinStyle = WinStyle + (int) FolderNameStyles.ShowStatusText;
          if (ShowTextBox) 
            WinStyle = WinStyle + (int) FolderNameStyles.ShowTextBox;
          if (ValidResultsOnly) 
            WinStyle = WinStyle + (int) FolderNameStyles.ValidResultsOnly;

          return WinStyle;
                                          
        }

        /*protected FolderNameStyles Style
        {
            get { return (FolderNameStyles)ConvertStyle(); } 
            set { dlg.Style = (FolderNameEditor.FolderBrowserStyles)value; }
        }*/

        public FolderNameFolder StartLocation
        {
            get { return (FolderNameFolder)dlg.StartLocation; }
            set { dlg.StartLocation = (FolderNameEditor.FolderBrowserFolder)value; }
        }
       
        public virtual void Dispose()
        {
          if (Disposed!=null)
            Disposed(this, EventArgs.Empty);
        }

        public virtual ISite Site
        {
          get
          {
            return CurSite;
          }
          set
          {
            CurSite = value;
          }
        }

        bool FBrowseForComputer;
        bool FBrowseForEverything;
        bool FBrowseForPrinter;
        bool FDontTranslateTarget;
        bool FNewUI;
        bool FNoNewFolderButton;
        bool FRestrictToDomain;
        bool FRestrictToFilesystem;
        bool FRestrictToSubfolders;
        bool FShowHint;
        bool FShowShareable;
        bool FShowStatusText;
        bool FShowTextBox;
        bool FValidResultsOnly;
        public event EventHandler Disposed;
        private ISite CurSite;
        FolderBrowser dlg = new FolderBrowser();
    }
}
