// ImageCombo: Contributed by Horst Veith [hveith@intercom.at]
// ComboBox with images and indention 
#region Copyright  2002-2003 The Genghis Group
/*
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation is IsRequired, as shown here:
 * 
 * Portions Copyright  2002-2003 The Genghis Group (http://www.genghisgroup.com/).
 * 
 * 2. No substantial portion of the source code of this library may be redistributed
 * without the express written permission of the copyright holders, where
 * "substantial" is defined as enough code to be recognizably from this library. 
*/
#endregion
#region History
/*
 * 8/18/02 - Initial Version
 */
#endregion

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Reflection;

namespace Genghis.Windows.Forms
{
    /// <summary>
    /// Summary description for ImageCombo.
    /// </summary>
    public class ImageCombo : System.Windows.Forms.ComboBox
    {
        public ImageCombo()
        {
            this.DrawMode = DrawMode.OwnerDrawFixed;
            this.DrawItem += new DrawItemEventHandler(OnDrawItem);
        }

        #region Properties

        [Description("The ImageList used to display the icons"), Category("Behavior")]
        public System.Windows.Forms.ImageList ImageList
        {
            get { return imageList; }
            set { imageList = value; }
        }

        [Description("Indicates the property to use as the actual indent for the item in the control"), 
         Category("Data")]
        public string IndentMember
        {
            get { return indentMember; }
            set { indentMember = value;    }
        }

        [Description("Indicates the property to use as the actual image index for the items in the control"), 
         Category("Data")]
        public string ImageMember
        {
            get { return imageMember; }
            set { imageMember = value;    }
        }

        [Description("Indicates the property to use as the actual image index for selected items in the control"), 
         Category("Data")]
        public string SelectedImageMember
        {
            get { return selectedImageMember; }
            set { selectedImageMember = value;    }
        }

        #endregion

        #region Implementation
        
        private void OnDrawItem(object sender, DrawItemEventArgs e)
        {
            e.DrawBackground();

            Rectangle rect = new Rectangle(e.Bounds.Location, e.Bounds.Size);
            object obj = this.Items[e.Index];

            int indent = GetParameter(obj, IndentMember);
            if (indent != -1)
                rect.Offset(indent, 0);
                        
            if (imageList != null)
            {
                int imageIndex = GetParameter(obj, this.ImageMember);
                if (e.Index == this.SelectedIndex)
                {
                    int selectedImage = GetParameter(obj, this.SelectedImageMember);
                    if (selectedImage != -1)
                        imageIndex = selectedImage;
                }

                if (imageIndex != -1)
                    imageList.Draw(e.Graphics, rect.Left, rect.Top, imageIndex);
                rect.Offset(imageList.ImageSize.Width, 0);
            }
            if (obj != null)
            {
                string str = string.Empty;
                if (this.DisplayMember != string.Empty)
                {
                    PropertyInfo propertyInfo = obj.GetType().GetProperty(this.DisplayMember);
                    if (propertyInfo != null)
                        str = propertyInfo.GetValue(obj, null).ToString();
                }
                else
                    str = obj.ToString();
                e.Graphics.DrawString(str, e.Font, new SolidBrush(e.ForeColor), rect);
            }
        }

        private int GetParameter(object obj, string propertyName)
        {
            if (obj == null || propertyName == string.Empty)
                return -1;

            PropertyInfo propertyInfo = obj.GetType().GetProperty(propertyName);
            if (propertyInfo != null)
            {
                object result = propertyInfo.GetValue(obj, null);
                if (result != null)
                    return Convert.ToInt32(result);
            }
            return -1;
        }

        private System.Windows.Forms.ImageList imageList = null;
        private string indentMember = string.Empty;
        private string imageMember = string.Empty;
        private string selectedImageMember = string.Empty;

        #endregion
    }
}
