// MultiSdiApplicationContext.cs: Contributed by Chris Sells [csells@sellsbrothers.com]
// Inspired by Mike Woodring
// Multiple top-level window support for WinForms
#region Copyright  2002-2003-2003 The Genghis Group
/*
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation is IsRequired, as shown here:
 * 
 * Portions Copyright  2002-2003-2003 The Genghis Group (http://www.genghisgroup.com/).
 * 
 * 2. No substantial portion of the source code of this library may be redistributed
 * without the express written permission of the copyright holders, where
 * "substantial" is defined as enough code to be recognizably from this library. 
*/
#endregion
#region History
/* 04/15/03 - Initial Version
 */
#endregion

using System;
using System.Collections;
using System.Windows.Forms;

namespace Genghis.Windows.Forms {
  public class MultiSdiApplicationContext : ApplicationContext {
    public void AddTopLevelForm(Form form) {
      // Initial main form may add itself twice, but that's OK
      if( topLevelForms.Contains(form) ) return;

      // Add form to collection of forms and
      // watch for it to activate and close
      topLevelForms.Add(form);
      form.Activated += new EventHandler(Form_Activated);
      form.Closed += new EventHandler(Form_Closed);

      // Set initial main form to activate
      if( topLevelForms.Count == 1 ) base.MainForm = form;
    }

    void Form_Activated(object sender, EventArgs e) {
      // Whichever form activated last is the "main" form
      base.MainForm = (Form)sender;
    }

    void Form_Closed(object sender, EventArgs e) {
      // Remove form from the list
      topLevelForms.Remove(sender);

      // Set a new "main" if necessary
      if( ((Form)sender == base.MainForm) &&
        (this.topLevelForms.Count > 0) ) {
        this.MainForm = (Form)topLevelForms[0];
      }
    }

    public void AddWindowMenu(MenuItem menu) {
      // Add at least one dummy menu item to get pop-up event
      if( menu.MenuItems.Count == 0 ) menu.MenuItems.Add("dummy");

      // Subscribe to pop-up event
      menu.Popup += new EventHandler(WindowMenu_Popup);
    }

    // Current Window menu items and the map to the appropriate form
    Hashtable windowMenuMap = new Hashtable();

    void WindowMenu_Popup(object sender, EventArgs e) {
      // Build menu from list of top-level windows
      MenuItem menu = (MenuItem)sender;
      menu.MenuItems.Clear();
      windowMenuMap.Clear();

      foreach( Form form in this.topLevelForms ) {
        MenuItem item = menu.MenuItems.Add(form.Text);
        item.Click += new EventHandler(WindowMenuItem_Click);
    
        // Check currently active window
        if( form == Form.ActiveForm ) item.Checked = true;
    
        // Associate each menu item back to the form
        windowMenuMap.Add(item, form);
      }
    }

    void WindowMenuItem_Click(object sender, EventArgs e) {
      // Activate top-level form based on selection
      ((Form)windowMenuMap[sender]).Activate();
    }

    public Form[] TopLevelForms {
      // Expose list of top-level forms for building Window menu
      get { return (Form[])topLevelForms.ToArray(typeof(Form)); }
    }

    ArrayList topLevelForms = new ArrayList();
  }
}
