#region Copyright  2002-2003 The Genghis Group
/*
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation is required, as shown here:
 *
 * Portions Copyright  2002-2003 The Genghis Group (http://www.genghisgroup.com/).
 *
 * 2. No substantial portion of the source code of this library may be redistributed
 * without the express written permission of the copyright holders, where
 * "substantial" is defined as enough code to be recognizably from this library.
*/
#endregion
#region History
/*
 * 3/30/03:
 * Initial Version
*/
#endregion

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Drawing.Drawing2D;

namespace Genghis.Windows.Forms
{
    public enum THEMESIZE
    {
        TS_MIN = 0,             // minimum size
        TS_TRUE = 1,            // size without stretching
        TS_DRAW = 2,            // size that theme mgr will use to draw part
    };

    public enum BGTYPE
    {
        BT_IMAGEFILE = 0,
        BT_BORDERFILL = 1,
        BT_NONE = 2
    }

    public enum IMAGELAYOUT
    {
        IL_VERTICAL = 0,
        IL_HORIZONTAL = 1
    }

    public enum BORDERTYPE
    {
        BT_RECT = 0,
        BT_ROUNDRECT = 1,
        BT_ELLIPSE = 2
    }

    public enum FILLTYPE
    {
        FT_SOLID = 0,
        FT_VERTGRADIENT = 1,
        FT_HORZGRADIENT = 2,
        FT_RADIALGRADIENT = 3,
        FT_TILEIMAGE = 4
    }

    public enum SIZINGTYPE
    {
        ST_TRUESIZE = 0,
        ST_STRETCH = 1,
        ST_TILE = 2
    }

    public enum HALIGN
    {
        HA_LEFT = 0,
        HA_CENTER = 1,
        HA_RIGHT = 2
    }

    public enum CONTENTALIGNMENT
    {
        CA_LEFT = 0,
        CA_CENTER = 1,
        CA_RIGHT = 2
    }

    public enum VALIGN
    {
        VA_TOP = 0,
        VA_CENTER = 1,
        VA_BOTTOM = 2
    }

    public enum OFFSETTYPE
    {
        OT_TOPLEFT = 0,
        OT_TOPRIGHT = 1,
        OT_TOPMIDDLE = 2,
        OT_BOTTOMLEFT = 3,
        OT_BOTTOMRIGHT = 4,
        OT_BOTTOMMIDDLE = 5,
        OT_MIDDLELEFT = 6,
        OT_MIDDLERIGHT = 7,
        OT_LEFTOFCAPTION = 8,
        OT_RIGHTOFCAPTION = 9,
        OT_LEFTOFLASTBUTTON = 10,
        OT_RIGHTOFLASTBUTTON = 11,
        OT_ABOVELASTBUTTON = 12,
        OT_BELOWLASTBUTTON = 13
    }

    public enum ICONEFFECT
    {
        ICE_NONE = 0,
        ICE_GLOW = 1,
        ICE_SHADOW = 2,
        ICE_PULSE = 3,
        ICE_ALPHA = 4
    }

    public enum TEXTSHADOWTYPE
    {
        TST_NONE = 0,
        TST_SINGLE = 1,
        TST_CONTINUOUS = 2
    }

    public enum GLYPHTYPE
    {
        GT_NONE = 0,
        GT_IMAGEGLYPH = 1,
        GT_FONTGLYPH = 2
    }

    public enum IMAGESELECTTYPE
    {
        IST_NONE = 0,
        IST_SIZE = 1,
        IST_DPI= 2
    }

    public enum TRUESIZESCALINGTYPE
    {
        TSST_NONE = 0,
        TSST_SIZE = 1,
        TSST_DPI = 2
    }

    public enum GLYPHFONTSIZINGTYPE
    {
        GFST_NONE = 0,
        GFST_SIZE = 1,
        GFST_DPI = 2
    }

    public enum TM_PROPS
    {
        TMT_STRING = 201,
        TMT_INT = 202,
        TMT_BOOL = 203,
        TMT_COLOR = 204,
        TMT_MARGINS = 205,
        TMT_FILENAME = 206,
        TMT_SIZE = 207,
        TMT_POSITION = 208,
        TMT_RECT = 209,
        TMT_FONT = 210,
        TMT_INTLIST = 211,

        TMT_COLORSCHEMES = 401,
        TMT_SIZES = 402,
        TMT_CHARSET = 403,

        TMT_DISPLAYNAME = 601,
        TMT_TOOLTIP = 602,
        TMT_COMPANY = 603,
        TMT_AUTHOR = 604,
        TMT_COPYRIGHT = 605,
        TMT_URL = 606,
        TMT_VERSION = 607,
        TMT_DESCRIPTION = 608,

        TMT_CAPTIONFONT = 801,
        TMT_SMALLCAPTIONFONT = 802,
        TMT_MENUFONT = 803,
        TMT_STATUSFONT = 804,
        TMT_MSGBOXFONT = 805,
        TMT_ICONTITLEFONT = 806,

        TMT_FLATMENUS = 1001,

        TMT_SIZINGBORDERWIDTH = 1201,
        TMT_SCROLLBARWIDTH = 1202,
        TMT_SCROLLBARHEIGHT = 1203,
        TMT_CAPTIONBARWIDTH = 1204,
        TMT_CAPTIONBARHEIGHT = 1205,
        TMT_SMCAPTIONBARWIDTH = 1206,
        TMT_SMCAPTIONBARHEIGHT = 1207,
        TMT_MENUBARWIDTH = 1208,
        TMT_MENUBARHEIGHT = 1209,

        TMT_MINCOLORDEPTH = 1301,

        TMT_CSSNAME = 1401,
        TMT_XMLNAME = 1402,

        TMT_SCROLLBAR = 1601,
        TMT_BACKGROUND = 1602,
        TMT_ACTIVECAPTION = 1603,
        TMT_INACTIVECAPTION = 1604,
        TMT_MENU = 1605,
        TMT_WINDOW = 1606,
        TMT_WINDOWFRAME = 1607,
        TMT_MENUTEXT = 1608,
        TMT_WINDOWTEXT = 1609,
        TMT_CAPTIONTEXT = 1610,
        TMT_ACTIVEBORDER = 1611,
        TMT_INACTIVEBORDER = 1612,
        TMT_APPWORKSPACE = 1613,
        TMT_HIGHLIGHT = 1614,
        TMT_HIGHLIGHTTEXT = 1615,
        TMT_BTNFACE = 1616,
        TMT_BTNSHADOW = 1617,
        TMT_GRAYTEXT = 1618,
        TMT_BTNTEXT = 1619,
        TMT_INACTIVECAPTIONTEXT = 1620,
        TMT_BTNHIGHLIGHT = 1621,
        TMT_DKSHADOW3D = 1622,
        TMT_LIGHT3D = 1623,
        TMT_INFOTEXT = 1624,
        TMT_INFOBK = 1625,
        TMT_BUTTONALTERNATEFACE = 1626,
        TMT_HOTTRACKING = 1627,
        TMT_GRADIENTACTIVECAPTION = 1628,
        TMT_GRADIENTINACTIVECAPTION = 1629,
        TMT_MENUHILIGHT = 1630,
        TMT_MENUBAR = 1631,

        TMT_FROMHUE1 = 1801,
        TMT_FROMHUE2 = 1802,
        TMT_FROMHUE3 = 1803,
        TMT_FROMHUE4 = 1804,
        TMT_FROMHUE5 = 1805,
        TMT_TOHUE1 = 1806,
        TMT_TOHUE2 = 1807,
        TMT_TOHUE3 = 1808,
        TMT_TOHUE4 = 1809,
        TMT_TOHUE5 = 1810,

        TMT_FROMCOLOR1 = 2001,
        TMT_FROMCOLOR2 = 2002,
        TMT_FROMCOLOR3 = 2003,
        TMT_FROMCOLOR4 = 2004,
        TMT_FROMCOLOR5 = 2005,
        TMT_TOCOLOR1 = 2006,
        TMT_TOCOLOR2 = 2007,
        TMT_TOCOLOR3 = 2008,
        TMT_TOCOLOR4 = 2009,
        TMT_TOCOLOR5 = 2010,

        TMT_TRANSPARENT = 2201,
        TMT_AUTOSIZE = 2202,
        TMT_BORDERONLY = 2203,
        TMT_COMPOSITED = 2204,
        TMT_BGFILL = 2205,
        TMT_GLYPHTRANSPARENT = 2206,
        TMT_GLYPHONLY = 2207,
        TMT_ALWAYSSHOWSIZINGBAR = 2208,
        TMT_MIRRORIMAGE = 2209,
        TMT_UNIFORMSIZING = 2210,
        TMT_INTEGRALSIZING = 2211,
        TMT_SOURCEGROW = 2212,
        TMT_SOURCESHRINK = 2213,

        TMT_IMAGECOUNT = 2401,
        TMT_ALPHALEVEL = 2402,
        TMT_BORDERSIZE = 2403,
        TMT_ROUNDCORNERWIDTH = 2404,
        TMT_ROUNDCORNERHEIGHT = 2405,
        TMT_GRADIENTRATIO1 = 2406,
        TMT_GRADIENTRATIO2 = 2407,
        TMT_GRADIENTRATIO3 = 2408,
        TMT_GRADIENTRATIO4 = 2409,
        TMT_GRADIENTRATIO5 = 2410,
        TMT_PROGRESSCHUNKSIZE = 2411,
        TMT_PROGRESSSPACESIZE = 2412,
        TMT_SATURATION = 2413,
        TMT_TEXTBORDERSIZE = 2414,
        TMT_ALPHATHRESHOLD = 2415,
        TMT_WIDTH = 2416,
        TMT_HEIGHT = 2417,
        TMT_GLYPHINDEX = 2418,
        TMT_TRUESIZESTRETCHMARK = 2419,
        TMT_MINDPI1 = 2420,
        TMT_MINDPI2 = 2421,
        TMT_MINDPI3 = 2422,
        TMT_MINDPI4 = 2423,
        TMT_MINDPI5 = 2424,

        TMT_GLYPHFONT = 2601,

        TMT_IMAGEFILE = 3001,
        TMT_IMAGEFILE1 = 3002,
        TMT_IMAGEFILE2 = 3003,
        TMT_IMAGEFILE3 = 3004,
        TMT_IMAGEFILE4 = 3005,
        TMT_IMAGEFILE5 = 3006,
        TMT_STOCKIMAGEFILE = 3007,
        TMT_GLYPHIMAGEFILE = 3008,

        TMT_TEXT = 3201,

        TMT_OFFSET = 3401,
        TMT_TEXTSHADOWOFFSET = 3402,
        TMT_MINSIZE = 3403,
        TMT_MINSIZE1 = 3404,
        TMT_MINSIZE2 = 3405,
        TMT_MINSIZE3 = 3406,
        TMT_MINSIZE4 = 3407,
        TMT_MINSIZE5 = 3408,
        TMT_NORMALSIZE = 3409,

        TMT_SIZINGMARGINS = 3601,
        TMT_CONTENTMARGINS = 3602,
        TMT_CAPTIONMARGINS = 3603,

        TMT_BORDERCOLOR = 3801,
        TMT_FILLCOLOR = 3802,
        TMT_TEXTCOLOR = 3803,
        TMT_EDGELIGHTCOLOR = 3804,
        TMT_EDGEHIGHLIGHTCOLOR = 3805,
        TMT_EDGESHADOWCOLOR = 3806,
        TMT_EDGEDKSHADOWCOLOR = 3807,
        TMT_EDGEFILLCOLOR = 3808,
        TMT_TRANSPARENTCOLOR = 3809,
        TMT_GRADIENTCOLOR1 = 3810,
        TMT_GRADIENTCOLOR2 = 3811,
        TMT_GRADIENTCOLOR3 = 3812,
        TMT_GRADIENTCOLOR4 = 3813,
        TMT_GRADIENTCOLOR5 = 3814,
        TMT_SHADOWCOLOR = 3815,
        TMT_GLOWCOLOR = 3816,
        TMT_TEXTBORDERCOLOR = 3817,
        TMT_TEXTSHADOWCOLOR = 3818,
        TMT_GLYPHTEXTCOLOR = 3819,
        TMT_GLYPHTRANSPARENTCOLOR = 3820,
        TMT_FILLCOLORHINT = 3821,
        TMT_BORDERCOLORHINT = 3822,
        TMT_ACCENTCOLORHINT = 3823,

        TMT_BGTYPE = 4001,
        TMT_BORDERTYPE = 4002,
        TMT_FILLTYPE = 4003,
        TMT_SIZINGTYPE = 4004,
        TMT_HALIGN = 4005,
        TMT_CONTENTALIGNMENT = 4006,
        TMT_VALIGN = 4007,
        TMT_OFFSETTYPE = 4008,
        TMT_ICONEFFECT = 4009,
        TMT_TEXTSHADOWTYPE = 4010,
        TMT_IMAGELAYOUT = 4011,
        TMT_GLYPHTYPE = 4012,
        TMT_IMAGESELECTTYPE = 4013,
        TMT_GLYPHFONTSIZINGTYPE = 4014,
        TMT_TRUESIZESCALINGTYPE = 4015,

        TMT_USERPICTURE = 5001,
        TMT_DEFAULTPANESIZE = 5002,
        TMT_BLENDCOLOR = 5003
    }
    
    public enum WINDOW_PARTS
    {
        WP_CAPTION = 1,
        WP_SMALLCAPTION = 2,
        WP_MINCAPTION = 3,
        WP_SMALLMINCAPTION = 4,
        WP_MAXCAPTION = 5,
        WP_SMALLMAXCAPTION = 6,
        WP_FRAMELEFT = 7,
        WP_FRAMERIGHT = 8,
        WP_FRAMEBOTTOM = 9,
        WP_SMALLFRAMELEFT = 10,
        WP_SMALLFRAMERIGHT = 11,
        WP_SMALLFRAMEBOTTOM = 12,

        WP_SYSBUTTON = 13,
        WP_MDISYSBUTTON = 14,
        WP_MINBUTTON = 15,
        WP_MDIMINBUTTON = 16,
        WP_MAXBUTTON = 17,
        WP_CLOSEBUTTON = 18,
        WP_SMALLCLOSEBUTTON = 19,
        WP_MDICLOSEBUTTON = 20,
        WP_RESTOREBUTTON = 21,
        WP_MDIRESTOREBUTTON = 22,
        WP_HELPBUTTON = 23,
        WP_MDIHELPBUTTON = 24,

        WP_HORZSCROLL = 25,
        WP_HORZTHUMB= 26,
        WP_VERTSCROLL = 27,
        WP_VERTTHUMB = 28,

        WP_DIALOG = 29,

        WP_CAPTIONSIZINGTEMPLATE = 30,
        WP_SMALLCAPTIONSIZINGTEMPLATE = 31,
        WP_FRAMELEFTSIZINGTEMPLATE = 32,
        WP_SMALLFRAMELEFTSIZINGTEMPLATE = 33,
        WP_FRAMERIGHTSIZINGTEMPLATE = 34,
        WP_SMALLFRAMERIGHTSIZINGTEMPLATE = 35,
        WP_FRAMEBOTTOMSIZINGTEMPLATE = 36,
        WP_SMALLFRAMEBOTTOMSIZINGTEMPLATE = 37
    }

    public enum FRAME_STATES
    {
        FS_ACTIVE = 1,
        FS_INACTIVE = 2
    }

    public enum CAPTION_STATES
    {
        CS_ACTIVE = 1,
        CS_INACTIVE = 2,
        CS_DISABLED = 3
    }
    
    public enum MAXCAPTION_STATES
    {
        MXCS_ACTIVE = 1,
        MXCS_INACTIVE = 2,
        MXCS_DISABLED = 3
    }

    public enum MINCAPTION_STATES
    {
        MNCS_ACTIVE = 1,
        MNCS_INACTIVE = 2,
        MNCS_DISABLED = 3
    }

    public enum HORZSCROLL_STATES
    {
        HSS_NORMAL = 1,
        HSS_HOT = 2,
        HSS_PUSHED = 3,
        HSS_DISABLED = 4
    }

    public enum HORZTHUMB_STATES
    {
        HTS_NORMAL = 1,
        HTS_HOT = 2,
        HTS_PUSHED = 3,
        HTS_DISABLED = 4
    }

    public enum VERTSCROLL_STATES
    {
        VSS_NORMAL = 1,
        VSS_HOT = 2,
        VSS_PUSHED = 3,
        VSS_DISABLED = 4
    }

    public enum VERTTHUMB_STATES
    {
        VTS_NORMAL = 1,
        VTS_HOT = 2,
        VTS_PUSHED = 3,
        VTS_DISABLED = 4
    }

    public enum SYSBUTTON_STATES
    {
        SBS_NORMAL = 1,
        SBS_HOT = 2,
        SBS_PUSHED = 3,
        SBS_DISABLED = 4
    }

    public enum MINBUTTON_STATES
    {
        MINBS_NORMAL = 1,
        MINBS_HOT = 2,
        MINBS_PUSHED = 3,
        MINBS_DISABLED = 4
    }

    public enum MAXBUTTON_STATES
    {
        MAXBS_NORMAL = 1,
        MAXBS_HOT = 2,
        MAXBS_PUSHED = 3,
        MAXBS_DISABLED = 4
    }

    public enum RESTOREBUTTON_STATES
    {
        RBS_NORMAL = 1,
        RBS_HOT = 2,
        RBS_PUSHED = 3,
        RBS_DISABLED = 4
    }

    public enum HELPBUTTON_STATES
    {
        HBS_NORMAL = 1,
        HBS_HOT = 2,
        HBS_PUSHED = 3,
        HBS_DISABLED = 4
    }

    public enum CLOSEBUTTON_STATES
    {
        CBS_NORMAL = 1,
        CBS_HOT = 2,
        CBS_PUSHED = 3,
        CBS_DISABLED = 4
    }

    public enum BUTTON_PARTS
    {
        BP_PUSHBUTTON = 1,
        BP_RADIOBUTTON = 2,
        BP_CHECKBOX = 3,
        BP_GROUPBOX = 4,
        BP_USERBUTTON = 5
    }

    public enum PUSHBUTTON_STATES
    {
        PBS_NORMAL = 1,
        PBS_HOT = 2,
        PBS_PRESSED = 3,
        PBS_DISABLED = 4,
        PBS_DEFAULTED = 5
    }

    public enum RADIOBUTTON_STATES
    {
        RBS_UNCHECKEDNORMAL = 1,
        RBS_UNCHECKEDHOT = 2,
        RBS_UNCHECKEDPRESSED = 3,
        RBS_UNCHECKEDDISABLED = 4,
        RBS_CHECKEDNORMAL = 5,
        RBS_CHECKEDHOT = 6,
        RBS_CHECKEDPRESSED = 7,
        RBS_CHECKEDDISABLED = 8
    }

    public enum CHECKBOX_STATES
    {
        CBS_UNCHECKEDNORMAL = 1,
        CBS_UNCHECKEDHOT = 2,
        CBS_UNCHECKEDPRESSED = 3,
        CBS_UNCHECKEDDISABLED = 4,
        CBS_CHECKEDNORMAL = 5,
        CBS_CHECKEDHOT = 6,
        CBS_CHECKEDPRESSED = 7,
        CBS_CHECKEDDISABLED = 8,
        CBS_MIXEDNORMAL = 9,
        CBS_MIXEDHOT = 10,
        CBS_MIXEDPRESSED = 11,
        CBS_MIXEDDISABLED = 12
    }

    public enum GROUPBOX_STATES
    {
        GBS_NORMAL = 1,
        GBS_DISABLED = 2
    }

    public enum REBAR_PARTS
    {
        RP_GRIPPER = 1,
        RP_GRIPPERVERT = 2,
        RP_BAND = 3,
        RP_CHEVRON = 4,
        RP_CHEVRONVERT = 5
    }

    public enum CHEVRON_STATES
    {
        CHEVS_NORMAL = 1,
        CHEVS_HOT = 2,
        CHEVS_PRESSED = 3
    }

    public enum TOOLBAR_PARTS
    {
        TP_BUTTON = 1,
        TP_DROPDOWNBUTTON = 2,
        TP_SPLITBUTTON = 3,
        TP_SPLITBUTTONDROPDOWN = 4,
        TP_SEPARATOR = 5,
        TP_SEPARATORVERT = 6
    }
    
    public enum TOOLBAR_STATES
    {
        TS_NORMAL = 1,
        TS_HOT = 2,
        TS_PRESSED = 3,
        TS_DISABLED = 4,
        TS_CHECKED = 5,
        TS_HOTCHECKED = 6
    }

    public enum STATUS_PARTS
    {
        SP_PANE = 1,
        SP_GRIPPERPANE = 2,
        SP_GRIPPER = 3
    }

    public enum MENU_PARTS
    {
        MP_MENUITEM = 1,
        MP_MENUDROPDOWN = 2,
        MP_MENUBARITEM = 3,
        MP_MENUBARDROPDOWN = 4,
        MP_CHEVRON = 5,
        MP_SEPARATOR = 6
    }

    public enum MENU_STATES
    {
        MS_NORMAL = 1,
        MS_SELECTED = 2,
        MS_DEMOTED = 3
    }

    public enum LISTVIEW_PARTS
    {
        LVP_LISTITEM = 1,
        LVP_LISTGROUP = 2,
        LVP_LISTDETAIL = 3,
        LVP_LISTSORTEDDETAIL = 4,
        LVP_EMPTYTEXT = 5
    }

    public enum LISTITEM_STATES
    {
        LIS_NORMAL = 1,
        LIS_HOT = 2,
        LIS_SELECTED = 3,
        LIS_DISABLED = 4,
        LIS_SELECTEDNOTFOCUS = 5
    }

    public enum HEADER_PARTS
    {
        HP_HEADERITEM = 1,
        HP_HEADERITEMLEFT = 2,
        HP_HEADERITEMRIGHT = 3,
        HP_HEADERSORTARROW = 4
    }

    public enum HEADERITEM_STATES
    {
        HIS_NORMAL = 1,
        HIS_HOT = 2,
        HIS_PRESSED = 3
    }

    public enum HEADERITEMLEFT_STATES
    {
        HILS_NORMAL = 1,
        HILS_HOT = 2,
        HILS_PRESSED = 3
    }

    public enum HEADERITEMRIGHT_STATES
    {
        HIRS_NORMAL = 1,
        HIRS_HOT = 2,
        HIRS_PRESSED = 3
    }

    public enum HEADERSORTARROW_STATES
    {
        HSAS_SORTEDUP = 1,
        HSAS_SORTEDDOWN = 2
    }

    public enum PROGRESS_PARTS
    {
        PP_BAR = 1,
        PP_BARVERT = 2,
        PP_CHUNK = 3,
        PP_CHUNKVERT = 4
    }

    public enum TAB_PARTS
    {
        TABP_TABITEM = 1,
        TABP_TABITEMLEFTEDGE = 2,
        TABP_TABITEMRIGHTEDGE = 3,
        TABP_TABITEMBOTHEDGE = 4,
        TABP_TOPTABITEM = 5,
        TABP_TOPTABITEMLEFTEDGE = 6,
        TABP_TOPTABITEMRIGHTEDGE = 7,
        TABP_TOPTABITEMBOTHEDGE = 8,
        TABP_PANE = 9,
        TABP_BODY = 10
    }

    public enum TABITEM_STATES
    {
        TIS_NORMAL = 1,
        TIS_HOT = 2,
        TIS_SELECTED = 3,
        TIS_DISABLED = 4,
        TIS_FOCUSED = 5
    }

    public enum TABITEMLEFTEDGE_STATES
    {
        TILES_NORMAL = 1,
        TILES_HOT = 2,
        TILES_SELECTED = 3,
        TILES_DISABLED = 4,
        TILES_FOCUSED = 5
    }

    public enum TABITEMRIGHTEDGE_STATES
    {
        TIRES_NORMAL = 1,
        TIRES_HOT = 2,
        TIRES_SELECTED = 3,
        TIRES_DISABLED = 4,
        TIRES_FOCUSED = 5
    }

    public enum TABITEMBOTHEDGES_STATES
    {
        TIBES_NORMAL = 1,
        TIBES_HOT = 2,
        TIBES_SELECTED = 3,
        TIBES_DISABLED = 4,
        TIBES_FOCUSED = 5
    }

    public enum TOPTABITEM_STATES
    {
        TTIS_NORMAL = 1,
        TTIS_HOT = 2,
        TTIS_SELECTED = 3,
        TTIS_DISABLED = 4,
        TTIS_FOCUSED = 5
    }

    public enum TOBTABITEMLEFTEDGE_STATES
    {
        TTILES_NORMAL = 1,
        TTILES_HOT = 2,
        TTILES_SELECTED = 3,
        TTILES_DISABLED = 4,
        TTILES_FOCUSED = 5
    }

    public enum TOBTABITEMRIGHTEDGE_STATES
    {
        TTIRES_NORMAL = 1,
        TTIRES_HOT = 2,
        TTIRES_SELECTED = 3,
        TTIRES_DISABLED = 4,
        TTIRES_FOCUSED = 5
    }

    public enum TOBTABITEMBOTHEDGES_STATES
    {
        TTIBES_NORMAL = 1,
        TTIBES_HOT = 2,
        TTIBES_SELECTED = 3,
        TTIBES_DISABLED = 4,
        TTIBES_FOCUSED = 5
    }

    public enum TRACKBAR_PARTS
    {
        TKP_TRACK = 1,
        TKP_TRACKVERT = 2,
        TKP_THUMB = 3,
        TKP_THUMBBOTTOM = 4,
        TKP_THUMBTOP = 5,
        TKP_THUMBVERT = 6,
        TKP_THUMBLEFT = 7,
        TKP_THUMBRIGHT = 8,
        TKP_TICS = 9,
        TKP_TICSVERT = 10
    }

    public enum TRACKBAR_STATES
    {
        TKS_NORMAL = 1
    }

    public enum TRACK_STATES
    {
        TRS_NORMAL = 1
    }

    public enum TRACKVERT_STATES
    {
        TRS_NORMAL = 1
    }

    public enum THUMB_STATES
    {
        TUS_NORMAL = 1,
        TUS_HOT = 2,
        TUS_PRESSED = 3,
        TUS_FOCUSED = 4,
        TUS_DISABLED = 5
    }

    public enum THUMBBOTTOM_STATES
    {
        TUBS_NORMAL = 1,
        TUBS_HOT = 2,
        TUBS_PRESSED = 3,
        TUBS_FOCUSED = 4,
        TUBS_DISABLED = 5
    }

    public enum THUMBTOP_STATES
    {
        TUTS_NORMAL = 1,
        TUTS_HOT = 2,
        TUTS_PRESSED = 3,
        TUTS_FOCUSED = 4,
        TUTS_DISABLED = 5
    }

    public enum THUMBVERT_STATES
    {
        TUVS_NORMAL = 1,
        TUVS_HOT = 2,
        TUVS_PRESSED = 3,
        TUVS_FOCUSED = 4,
        TUVS_DISABLED = 5
    }

    public enum THUMBLEFT_STATES
    {
        TUVLS_NORMAL = 1,
        TUVLS_HOT = 2,
        TUVLS_PRESSED = 3,
        TUVLS_FOCUSED = 4,
        TUVLS_DISABLED = 5
    }

    public enum THUMBRIGHT_STATES
    {
        TUVRS_NORMAL = 1,
        TUVRS_HOT = 2,
        TUVRS_PRESSED = 3,
        TUVRS_FOCUSED = 4,
        TUVRS_DISABLED = 5
    }

    public enum TICS_STATES
    {
        TSS_NORMAL = 1
    }

    public enum TICSVERT_STATES
    {
        TSVS_NORMAL = 1
    }
    
    public enum TOOLTIP_PARTS
    {
        TTP_STANDARD = 1,
        TTP_STANDARDTITLE = 2,
        TTP_BALLOON = 3,
        TTP_BALLOONTITLE = 4,
        TTP_CLOSE = 5
    }

    public enum CLOSE_STATES
    {
        TTCS_NORMAL = 1,
        TTCS_HOT = 2,
        TTCS_PRESSED = 3
    }

    public enum STANDARD_STATES
    {
        TTSS_NORMAL = 1,
        TTSS_LINK = 2
    }

    public enum BALLOON_STATES
    {
        TTBS_NORMAL = 1,
        TTBS_LINK = 2
    }

    public enum TREEVIEW_PARTS
    {
        TVP_TREEITEM = 1,
        TVP_GLYPH = 2,
        TVP_BRANCH = 3
    }

    public enum TREEITEM_STATES
    {
        TREIS_NORMAL = 1,
        TREIS_HOT = 2,
        TREIS_SELECTED = 3,
        TREIS_DISABLED = 4,
        TREIS_SELECTEDNOTFOCUS = 5
    }

    public enum GLYPH_STATES
    {
        GLPS_CLOSED = 1,
        GLPS_OPENED = 2
    }

    public enum SPIN_PARTS
    {
        SPNP_UP = 1,
        SPNP_DOWN = 2,
        SPNP_UPHORZ = 3,
        SPNP_DOWNHORZ = 4
    }

    public enum UP_STATES
    {
        UPS_NORMAL = 1,
        UPS_HOT = 2,
        UPS_PRESSED = 3,
        UPS_DISABLED = 4
    }

    public enum DOWN_STATES
    {
        DNS_NORMAL = 1,
        DNS_HOT = 2,
        DNS_PRESSED = 3,
        DNS_DISABLED = 4
    }

    public enum UPHORZ_STATES
    {
        UPHZS_NORMAL = 1,
        UPHZS_HOT = 2,
        UPHZS_PRESSED = 3,
        UPHZS_DISABLED = 4
    }

    public enum DOWNHORZ_STATES
    {
        DNHZS_NORMAL = 1,
        DNHZS_HOT = 2,
        DNHZS_PRESSED = 3,
        DNHZS_DISABLED = 4
    }

    public enum PAGE_PARTS
    {
        PGRP_UP = 1,
        PGRP_DOWN = 2,
        PGRP_UPHORZ = 3,
        PGRP_DOWNHORZ = 4
    }

    public enum SCROLLBAR_PARTS
    {
        SBP_ARROWBTN = 1,
        SBP_THUMBBTNHORZ = 2,
        SBP_THUMBBTNVERT = 3,
        SBP_LOWERTRACKHORZ = 4,
        SBP_UPPERTRACKHORZ = 5,
        SBP_LOWERTRACKVERT = 6,
        SBP_UPPERTRACKVERT = 7,
        SBP_GRIPPERHORZ = 8,
        SBP_GRIPPERVERT = 9,
        SBP_SIZEBOX = 10
    }

    public enum ARROWBTN_STATES
    {
        ABS_UPNORMAL = 1,
        ABS_UPHOT = 2,
        ABS_UPPRESSED = 3,
        ABS_UPDISABLED = 4,
        ABS_DOWNNORMAL = 5,
        ABS_DOWNHOT = 6,
        ABS_DOWNPRESSED = 7,
        ABS_DOWNDISABLED = 8,
        ABS_LEFTNORMAL = 9,
        ABS_LEFTHOT = 10,
        ABS_LEFTPRESSED = 11,
        ABS_LEFTDISABLED = 12,
        ABS_RIGHTNORMAL = 13,
        ABS_RIGHTHOT = 14,
        ABS_RIGHTPRESSED = 15,
        ABS_RIGHTDISABLED = 16
    }

    public enum SCROLLBAR_STATES
    {
        SCRBS_NORMAL = 1,
        SCRBS_HOT = 2,
        SCRBS_PRESSED = 3,
        SCRBS_DISABLED = 4
    }

    public enum SIZEBOX_STATES
    {
        SZB_RIGHTALIGN = 1,
        SZB_LEFTALIGN = 2
    }

    public enum EDIT_PARTS
    {
        EP_EDITTEXT = 1,
        EP_CARET = 2
    }

    public enum EDITTEXT_STATES
    {
        ETS_NORMAL = 1,
        ETS_HOT = 2,
        ETS_SELECTED = 3,
        ETS_DISABLED = 4,
        ETS_FOCUSED = 5,
        ETS_READONLY = 6,
        ETS_ASSIST = 7
    }

    public enum COMBOBOX_PARTS
    {
        CP_DROPDOWNBUTTON = 1
    }

    public enum COMBOBOX_STATES
    {
        CBXS_NORMAL = 1,
        CBXS_HOT = 2,
        CBXS_PRESSED = 3,
        CBXS_DISABLED = 4
    }

    public enum CLOCK_PARTS
    {
        CLP_TIME = 1
    }

    public enum CLOCK_STATES
    {
        CLS_NORMAL = 1
    }

    public enum TRAYNOTIFY_PARTS
    {
        TNP_BACKGROUND = 1,
        TNP_ANIMBACKGROUND = 2
    }

    public enum TASKBAR_PARTS
    {
        TBP_BACKGROUNDBOTTOM = 1,
        TBP_BACKGROUNDRIGHT = 2,
        TBP_BACKGROUNDTOP = 3,
        TBP_BACKGROUNDLEFT = 4,
        TBP_SIZINGBARBOTTOM = 5,
        TBP_SIZINGBARRIGHT = 6,
        TBP_SIZINGBARTOP = 7,
        TBP_SIZINGBARLEFT = 8
    }

    public enum TASKBAND_PARTS
    {
        TDP_GROUPCOUNT = 1,
        TDP_FLASHBUTTON = 2,
        TDP_FLASHBUTTONGROUPMENU = 3
    }

    public enum STARTPANEL_PARTS
    {
        SPP_USERPANE = 1,
        SPP_MOREPROGRAMS = 2,
        SPP_MOREPROGRAMSARROW = 3,
        SPP_PROGLIST = 4,
        SPP_PROGLISTSEPARATOR = 5,
        SPP_PLACESLIST = 6,
        SPP_PLACESLISTSEPARATOR = 7,
        SPP_LOGOFF = 8,
        SPP_LOGOFFBUTTONS = 9,
        SPP_USERPICTURE = 10,
        SPP_PREVIEW = 11
    }

    public enum MOREPROGRAMSARROW_STATES
    {
        SPS_NORMAL = 1,
        SPS_HOT = 2,
        SPS_PRESSED = 3
    }

    public enum LOGOFFBUTTONS_STATES
    {
        SPLS_NORMAL = 1,
        SPLS_HOT = 2,
        SPLS_PRESSED = 3
    }

    public enum EXPLORERBAR_PARTS
    {
        EBP_HEADERBACKGROUND = 1,
        EBP_HEADERCLOSE = 2,
        EBP_HEADERPIN = 3,
        EBP_IEBARMENU = 4,
        EBP_NORMALGROUPBACKGROUND = 5,
        EBP_NORMALGROUPCOLLAPSE = 6,
        EBP_NORMALGROUPEXPAND = 7,
        EBP_NORMALGROUPHEAD = 8,
        EBP_SPECIALGROUPBACKGROUND = 9,
        EBP_SPECIALGROUPCOLLAPSE = 10,
        EBP_SPECIALGROUPEXPAND = 11,
        EBP_SPECIALGROUPHEAD = 12
    }

    public enum HEADERCLOSE_STATES
    {
        EBHC_NORMAL = 1,
        EBHC_HOT = 2,
        EBHC_PRESSED = 3
    }

    public enum HEADERPIN_STATES
    {
        EBHP_NORMAL = 1,
        EBHP_HOT = 2,
        EBHP_PRESSED = 3,
        EBHP_SELECTEDNORMAL = 4,
        EBHP_SELECFTEDHOT = 5,
        EBHP_SELECTEDHOTPRESSED = 6
    }

    public enum IEBARMENU_STATES
    {
        EBM_NORMAL = 1,
        EBM_HOT = 2,
        EBM_PRESSED = 3
    }

    public enum NORMALGROUPCOLLAPSE_STATES
    {
        EBNGC_NORMAL = 1,
        EBGNC_HOT = 2,
        EBGNC_PRESSED = 3
    }

    public enum NORMALGROUPEXPAND_STATES
    {
        EBNGE_NORMAL = 1,
        EBNGE_HOT = 2,
        EBNGE_PRESSED = 3
    }

    public enum SPECIALGROUPCOLLAPSE_STATES
    {
        EBSGC_NORMAL = 1,
        EBSGC_HOT = 2,
        EBSGC_PRESSED = 3
    }

    public enum SPECIALGROUPEXPAND_STATES
    {
        EBSGE_NORMAL = 1,
        EBSGE_HOT = 2,
        EBSGE_PRESSED = 3
    }

    public enum MENUBAND_PARTS
    {
        MDP_NEWAPPBUTTON = 1,
        MDP_SEPERATOR = 2
    }

    public enum MENUBAND_STATES
    {
        MDS_NORMAL = 1,
        MDS_HOT = 2,
        MDS_PRESSED = 3,
        MDS_DISABLED = 4,
        MDS_CHECKED = 5,
        MDS_HOTCHECKED
    }

    /// <summary>
    /// Paints a themed button
    /// </summary>
    public class ThemedButton : System.Windows.Forms.Button
    {
        bool mouseDown = false;
        bool mouseOver = false;
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedButton()
        {
            SetStyle( ControlStyles.AllPaintingInWmPaint | 
                ControlStyles.SupportsTransparentBackColor, true );
            mgr = new XPThemeManager( this );
        }

        /// <summary>
        /// Paint background
        /// </summary>
        /// <param name="pe">Event args</param>
        protected override void OnPaintBackground( PaintEventArgs pe )
        {
            if( !XPThemeManager.IsAppThemed || BackColor.A == 255 )
            {
                base.OnPaintBackground( pe );
            }
            else
            {
                mgr.OpenThemeData( "BUTTON" );
                mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );
                mgr.CloseThemeData();
            }

        }

        /// <summary>
        /// Paint control
        /// </summary>
        /// <param name="pe">Event args</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            if( XPThemeManager.IsAppThemed )
            {
                OnPaintBackground( pe );
                PUSHBUTTON_STATES state = PUSHBUTTON_STATES.PBS_NORMAL;
                if( !this.Enabled )
                {
                    state = PUSHBUTTON_STATES.PBS_DISABLED;
                }
                else 
                {    
                    Rectangle rcl = RectangleToScreen( ClientRectangle );

                    if( this.IsDefault )
                    {
                        state = PUSHBUTTON_STATES.PBS_DEFAULTED;
                    }
                    if( mouseDown && mouseOver && rcl.Contains( Control.MousePosition ) )
                    {
                        state = PUSHBUTTON_STATES.PBS_PRESSED;
                    }
                    else if( (mouseOver || mouseDown) && rcl.Contains( Control.MousePosition ))
                    {
                        state = PUSHBUTTON_STATES.PBS_HOT;
                    }
                }
                try
                {
                    mgr.OpenThemeData( "BUTTON" );
                    
                    mgr.DrawThemeBackground( pe.Graphics,
                        (int)BUTTON_PARTS.BP_PUSHBUTTON,
                        (int)state,
                        ClientRectangle,
                        pe.ClipRectangle );

                    Rectangle rcl = mgr.GetThemeBackgroundContentRect( pe.Graphics,
                        (int)BUTTON_PARTS.BP_PUSHBUTTON,
                        (int)state,
                        ClientRectangle );
                    Rectangle content = rcl;

                    StringFormat format = GetStringFormat();
                    using( Brush brush = new SolidBrush( ForeColor ) )
                    {
                        if( mouseDown && mouseOver )
                        {
                            rcl.Offset( 1, 1 );
                        }

                        if( ((ImageAlign == ContentAlignment.BottomLeft) ||
                            (ImageAlign == ContentAlignment.MiddleLeft) ||
                            (ImageAlign == ContentAlignment.TopLeft)) &&
                            ((TextAlign == ContentAlignment.BottomLeft) ||
                            (TextAlign == ContentAlignment.MiddleLeft) ||
                            (TextAlign == ContentAlignment.TopLeft)) )
                        {
                            rcl.X += Image.Width + 1;
                        }

                        if( ((ImageAlign == ContentAlignment.BottomRight) ||
                            (ImageAlign == ContentAlignment.MiddleRight) ||
                            (ImageAlign == ContentAlignment.TopRight)) &&
                            ((TextAlign == ContentAlignment.BottomRight) ||
                            (TextAlign == ContentAlignment.MiddleRight) ||
                            (TextAlign == ContentAlignment.TopRight)) )
                        {
                            rcl.X -= Image.Width + 1;
                        }

                        if( Enabled )
                        {
                            pe.Graphics.DrawString( Text, Font, brush, rcl, format );
                        }
                        else
                        {
                            ControlPaint.DrawStringDisabled( pe.Graphics, Text, Font, BackColor, rcl, format );
                        }
                    }
                    if( this.Image != null )
                    {
                        PaintImage( pe, (int)state );
                    }

                    if( this.Focused )
                    {
                        ControlPaint.DrawFocusRectangle( pe.Graphics, content );
                    }

                    mgr.CloseThemeData();
                }
                catch( Exception e )
                {
                    MessageBox.Show( e.ToString() );
                    base.OnPaint(pe);
                }
            }
            else
            {
                Region reg = new System.Drawing.Region();
                reg.MakeInfinite();
                Region = reg;
                base.OnPaint(pe);
            }
        }

        private void PaintImage( PaintEventArgs pe, int state )
        {
            Rectangle destRect = mgr.GetThemeBackgroundContentRect( pe.Graphics,
                (int)BUTTON_PARTS.BP_PUSHBUTTON,
                state,
                ClientRectangle );
            switch( ImageAlign )
            {
                case ContentAlignment.BottomCenter:
                    destRect.Y = destRect.Bottom - Image.Height;
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.BottomLeft:
                    destRect.Y = destRect.Bottom - Image.Height;
                    break;
                case ContentAlignment.BottomRight:
                    destRect.Y = destRect.Bottom - Image.Height;
                    destRect.X = destRect.Right - Image.Width;
                    break;
                case ContentAlignment.MiddleCenter:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.MiddleLeft:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    break;
                case ContentAlignment.MiddleRight:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    destRect.X = destRect.Right - Image.Width;
                    break;
                case ContentAlignment.TopCenter:
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.TopLeft:
                    break;
                case ContentAlignment.TopRight:
                    destRect.X = destRect.Right - Image.Width;
                    break;
                default:
                    break;
            }
            pe.Graphics.DrawImageUnscaled( this.Image, destRect );
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs mevent)
        {
            mouseDown = false;
            base.OnMouseUp( mevent );
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs mevent)
        {
            mouseDown = true;
            base.OnMouseDown( mevent );
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseEnter(System.EventArgs eventargs)
        {
            mouseOver = true;
            base.OnMouseEnter( eventargs );
            Invalidate();
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseLeave(System.EventArgs eventargs)
        {
            mouseOver = false;
            base.OnMouseLeave( eventargs );
            Invalidate();
        }

        private StringFormat GetStringFormat()
        {
            StringFormat format = new StringFormat();
            switch( this.TextAlign )
            {
                case ContentAlignment.BottomCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.BottomLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.BottomRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.MiddleCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.TopCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                default:
                    break;
            }
            return format;
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs mevent)
        {
            base.OnMouseMove( mevent );
            if( this.ClientRectangle.Contains( mevent.X, mevent.Y ) )
            {
                if( !mouseOver )
                {
                    mouseOver = true;
                    Invalidate();
                }
            }
            else
            {
                if( mouseOver )
                {
                    mouseOver = false;
                    Invalidate();
                }
            }
        }
    }

    /// <summary>
    /// Draws a themed check box, with transparent label
    /// </summary>
    public class ThemedCheckBox : System.Windows.Forms.CheckBox
    {
        bool mouseDown = false;
        bool mouseOver = false;
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedCheckBox()
        {
            SetStyle( ControlStyles.AllPaintingInWmPaint | 
                ControlStyles.SupportsTransparentBackColor, true );
            mgr = new XPThemeManager( this );
        }

        /// <summary>
        /// Paint background
        /// </summary>
        /// <param name="pe">event args</param>
        protected override void OnPaintBackground( PaintEventArgs pe )
        {
            if( !XPThemeManager.IsAppThemed || BackColor.A == 255 )
            {
                base.OnPaintBackground( pe );
            }
            else
            {
                mgr.OpenThemeData( "BUTTON" );
                mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );
                mgr.CloseThemeData();
            }
        }

        /// <summary>
        /// Paint control
        /// </summary>
        /// <param name="pe">event args</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            CHECKBOX_STATES state = CHECKBOX_STATES.CBS_MIXEDDISABLED;

            if( XPThemeManager.IsAppThemed )
            {
                OnPaintBackground( pe );
                switch( this.CheckState )
                {
                    case CheckState.Checked:
                        if( !Enabled )
                        {
                            state = CHECKBOX_STATES.CBS_CHECKEDDISABLED;
                        }
                        else if( mouseOver && mouseDown )
                        {
                            state = CHECKBOX_STATES.CBS_CHECKEDPRESSED;
                        }
                        else if( mouseOver || mouseDown )
                        {
                            state = CHECKBOX_STATES.CBS_CHECKEDHOT;
                        }
                        else
                        {
                            state = CHECKBOX_STATES.CBS_CHECKEDNORMAL;
                        }
                        break;
                    case CheckState.Unchecked:
                        if( !Enabled )
                        {
                            state = CHECKBOX_STATES.CBS_UNCHECKEDDISABLED;
                        }
                        else if( mouseOver && mouseDown )
                        {
                            state = CHECKBOX_STATES.CBS_UNCHECKEDPRESSED;
                        }
                        else if( mouseOver || mouseDown )
                        {
                            state = CHECKBOX_STATES.CBS_UNCHECKEDHOT;
                        }
                        else
                        {
                            state = CHECKBOX_STATES.CBS_UNCHECKEDNORMAL;
                        }
                        break;
                    case CheckState.Indeterminate:
                        if( !Enabled )
                        {
                            state = CHECKBOX_STATES.CBS_MIXEDDISABLED;
                        }
                        else if( mouseOver && mouseDown )
                        {
                            state = CHECKBOX_STATES.CBS_MIXEDPRESSED;
                        }
                        else if( mouseOver || mouseDown )
                        {
                            state = CHECKBOX_STATES.CBS_MIXEDHOT;
                        }
                        else
                        {
                            state = CHECKBOX_STATES.CBS_MIXEDNORMAL;
                        }
                        break;
                    default:
                        break;
                }

                try
                {
                    mgr.OpenThemeData( "BUTTON" );

                    mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );

                    Rectangle target = GetBackgroundRectangle(    mgr.GetThemePartSize( pe.Graphics, 
                        (int)BUTTON_PARTS.BP_CHECKBOX,
                        (int)state,
                        ClientRectangle, THEMESIZE.TS_DRAW ) );
                    mgr.DrawThemeBackground( pe.Graphics,
                        (int)BUTTON_PARTS.BP_CHECKBOX,
                        (int)state,
                        target,
                        pe.ClipRectangle );

                    Rectangle rcl = mgr.GetThemeBackgroundContentRect( pe.Graphics,
                        (int)BUTTON_PARTS.BP_CHECKBOX,
                        (int)state,
                        ClientRectangle );

                    if( ((CheckAlign == ContentAlignment.BottomLeft) ||
                        (CheckAlign == ContentAlignment.MiddleLeft) ||
                        (CheckAlign == ContentAlignment.TopLeft)) &&
                        ((TextAlign == ContentAlignment.BottomLeft) ||
                        (TextAlign == ContentAlignment.MiddleLeft) ||
                        (TextAlign == ContentAlignment.TopLeft)) )
                    {
                        rcl.X = target.Right + 1;
                    }

                    if( ((CheckAlign == ContentAlignment.BottomRight) ||
                        (CheckAlign == ContentAlignment.MiddleRight) ||
                        (CheckAlign == ContentAlignment.TopRight)) &&
                        ((TextAlign == ContentAlignment.BottomRight) ||
                        (TextAlign == ContentAlignment.MiddleRight) ||
                        (TextAlign == ContentAlignment.TopRight)) )
                    {
                        rcl.X -= target.Width + 1;
                    }

                    if( ((ImageAlign == ContentAlignment.BottomLeft) ||
                        (ImageAlign == ContentAlignment.MiddleLeft) ||
                        (ImageAlign == ContentAlignment.TopLeft)) &&
                        ((TextAlign == ContentAlignment.BottomLeft) ||
                        (TextAlign == ContentAlignment.MiddleLeft) ||
                        (TextAlign == ContentAlignment.TopLeft)) )
                    {
                        rcl.X += Image.Width + 1;
                    }

                    if( ((ImageAlign == ContentAlignment.BottomRight) ||
                        (ImageAlign == ContentAlignment.MiddleRight) ||
                        (ImageAlign == ContentAlignment.TopRight)) &&
                        ((TextAlign == ContentAlignment.BottomRight) ||
                        (TextAlign == ContentAlignment.MiddleRight) ||
                        (TextAlign == ContentAlignment.TopRight)) )
                    {
                        rcl.X -= Image.Width + 1;
                    }

                    StringFormat format = GetStringFormat();
                    using( Brush brush = new SolidBrush( ForeColor ) )
                    {
                        if( Enabled )
                        {
                            pe.Graphics.DrawString( Text, Font, brush, rcl, format );
                        }
                        else
                        {
                            ControlPaint.DrawStringDisabled( pe.Graphics, Text, Font, BackColor, rcl, format );
                        }
                    }
                    if( this.Image != null )
                    {
                        PaintImage( pe, (int)state, target );
                    }
                    mgr.CloseThemeData();
                }
                catch( Exception e )
                {
                    MessageBox.Show( e.ToString() );
                    base.OnPaint(pe);
                }
            }
            else
            {
                Region reg = new System.Drawing.Region();
                reg.MakeInfinite();
                Region = reg;
                base.OnPaint(pe);
            }
        }

        private void PaintImage( PaintEventArgs pe, int state, Rectangle target )
        {
            Rectangle destRect = mgr.GetThemeBackgroundContentRect( pe.Graphics,
                (int)BUTTON_PARTS.BP_CHECKBOX,
                state,
                ClientRectangle );
            switch( ImageAlign )
            {
                case ContentAlignment.BottomCenter:
                    destRect.Y = destRect.Bottom - Image.Height;
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.BottomLeft:
                    destRect.Y = destRect.Bottom - Image.Height;
                    break;
                case ContentAlignment.BottomRight:
                    destRect.Y = destRect.Bottom - Image.Height;
                    destRect.X = destRect.Right - Image.Width;
                    break;
                case ContentAlignment.MiddleCenter:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.MiddleLeft:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    break;
                case ContentAlignment.MiddleRight:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    destRect.X = destRect.Right - Image.Width;
                    break;
                case ContentAlignment.TopCenter:
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.TopLeft:
                    break;
                case ContentAlignment.TopRight:
                    destRect.X = destRect.Right - Image.Width;
                    break;
                default:
                    break;
            }
        
            if( ((CheckAlign == ContentAlignment.BottomLeft) ||
                (CheckAlign == ContentAlignment.MiddleLeft) ||
                (CheckAlign == ContentAlignment.TopLeft)) &&
                ((ImageAlign == ContentAlignment.BottomLeft) ||
                (ImageAlign == ContentAlignment.MiddleLeft) ||
                (ImageAlign == ContentAlignment.TopLeft)) )
            {
                destRect.X += target.Width + 1;
            }

            if( ((CheckAlign == ContentAlignment.BottomRight) ||
                (CheckAlign == ContentAlignment.MiddleRight) ||
                (CheckAlign == ContentAlignment.TopRight)) &&
                ((ImageAlign == ContentAlignment.BottomRight) ||
                (ImageAlign == ContentAlignment.MiddleRight) ||
                (ImageAlign == ContentAlignment.TopRight)) )
            {
                destRect.X -= target.Width + 1;
            }
            pe.Graphics.DrawImageUnscaled( this.Image, destRect );
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs mevent)
        {
            mouseDown = false;
            base.OnMouseUp( mevent );
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs mevent)
        {
            mouseDown = true;
            base.OnMouseDown( mevent );
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseEnter(System.EventArgs eventargs)
        {
            mouseOver = true;
            base.OnMouseEnter( eventargs );
            Invalidate();
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseLeave(System.EventArgs eventargs)
        {
            mouseOver = false;
            base.OnMouseLeave( eventargs );
            Invalidate();
        }

        private StringFormat GetStringFormat()
        {
            StringFormat format = new StringFormat();
            switch( this.TextAlign )
            {
                case ContentAlignment.BottomCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.BottomLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.BottomRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.MiddleCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.TopCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                default:
                    break;
            }
            return format;
        }

        private Rectangle GetBackgroundRectangle( Size size )
        {
            Rectangle destRect = new Rectangle( new Point( 0,0 ), size );
            switch( this.CheckAlign )
            {
                case ContentAlignment.BottomCenter:
                    destRect.Y = ClientRectangle.Bottom - size.Height;
                    destRect.X = destRect.Left + ((ClientRectangle.Width - size.Width) / 2);
                    break;
                case ContentAlignment.BottomLeft:
                    destRect.Y = ClientRectangle.Bottom - size.Height;
                    break;
                case ContentAlignment.BottomRight:
                    destRect.Y = ClientRectangle.Bottom - size.Height;
                    destRect.X = ClientRectangle.Right - size.Width;
                    break;
                case ContentAlignment.MiddleCenter:
                    destRect.Y = destRect.Top + ((ClientRectangle.Height - size.Height) / 2);
                    destRect.X = destRect.Left + ((ClientRectangle.Width - size.Width) / 2);
                    break;
                case ContentAlignment.MiddleLeft:
                    destRect.Y = destRect.Top + ((ClientRectangle.Height - size.Height) / 2);
                    break;
                case ContentAlignment.MiddleRight:
                    destRect.Y = destRect.Top + ((ClientRectangle.Height - size.Height) / 2);
                    destRect.X = ClientRectangle.Right - size.Width;
                    break;
                case ContentAlignment.TopCenter:
                    destRect.X = destRect.Left + ((ClientRectangle.Width - size.Width) / 2);
                    break;
                case ContentAlignment.TopLeft:
                    break;
                case ContentAlignment.TopRight:
                    destRect.X = ClientRectangle.Right - size.Width;
                    break;
                default:
                    break;
            }
            return destRect;
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs mevent)
        {
            base.OnMouseMove( mevent );
            if( this.ClientRectangle.Contains( mevent.X, mevent.Y ) )
            {
                if( !mouseOver )
                {
                    mouseOver = true;
                    Invalidate();
                }
            }
            else
            {
                if( mouseOver )
                {
                    mouseOver = false;
                    Invalidate();
                }
            }
        }
    }

    /// <summary>
    /// Draws a themed (transparent) groupbox
    /// </summary>
    public class ThemedGroupBox : System.Windows.Forms.GroupBox
    {
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedGroupBox()
        {
            SetStyle( 
                ControlStyles.SupportsTransparentBackColor |
                ControlStyles.UserPaint |
                ControlStyles.DoubleBuffer, true );
            mgr = new XPThemeManager( this );
        }

        /// <summary>
        /// Background paint
        /// </summary>
        /// <param name="pe">event args</param>
        protected override void OnPaintBackground( PaintEventArgs pe )
        {
            if( !XPThemeManager.IsAppThemed || BackColor.A == 255 )
            {
                base.OnPaintBackground( pe );
            }
            else
            {
                mgr.OpenThemeData( "BUTTON" );
                mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );
                mgr.CloseThemeData();
            }
        }

        /// <summary>
        /// Paint
        /// </summary>
        /// <param name="pe">Event args</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            if( XPThemeManager.IsAppThemed )
            {
                OnPaintBackground( pe );
                GROUPBOX_STATES state = GROUPBOX_STATES.GBS_NORMAL;
                if( !this.Enabled )
                {
                    state = GROUPBOX_STATES.GBS_DISABLED;
                }
                try
                {
                    mgr.OpenThemeData( "BUTTON" );
                    Rectangle rcl = ClientRectangle;

                    StringFormat format = GetStringFormat();
                    format.Trimming = StringTrimming.Character;
                    format.Alignment = StringAlignment.Near;
                    Rectangle clientRect = this.ClientRectangle;
                    clientRect.X += 8;
                    clientRect.Width -= 16;

                    if( this.RightToLeft == RightToLeft.Yes )
                    {
                        format.FormatFlags = format.FormatFlags | StringFormatFlags.DirectionRightToLeft;
                    }

                    SizeF stringSize = pe.Graphics.MeasureString( Text, Font, clientRect.Size, format );

                    RectangleF textRect;

                    if( this.RightToLeft == RightToLeft.No )
                    {
                        textRect = new RectangleF( new PointF(rcl.Left + 8f, rcl.Top ), stringSize );
                    }
                    else
                    {
                        textRect = new RectangleF( new PointF(rcl.Left + rcl.Width - 8 - stringSize.Width, rcl.Top ), stringSize );
                    }
                    Rectangle totalRect = new Rectangle( rcl.Left, rcl.Top + (Font.Height / 2), rcl.Width, rcl.Height - (Font.Height / 2) );

                    Rectangle clip1 = Rectangle.FromLTRB( rcl.Left, rcl.Top, (int)textRect.Left, (int)textRect.Bottom );
                    clip1.Intersect( pe.ClipRectangle );
                    Rectangle clip2 = Rectangle.FromLTRB( (int)textRect.Right, rcl.Top, rcl.Right, (int)textRect.Bottom );
                    clip2.Intersect( pe.ClipRectangle );
                    Rectangle clip3 = Rectangle.FromLTRB( rcl.Left, (int)textRect.Bottom, rcl.Right, rcl.Bottom );
                    clip3.Intersect( pe.ClipRectangle );

                    rcl.Y += Font.Height / 2;
                    rcl.Height -= Font.Height / 2;

                    if( !clip1.IsEmpty )
                    {
                        mgr.DrawThemeBackground( pe.Graphics,
                            (int)BUTTON_PARTS.BP_GROUPBOX,
                            (int)state,
                            rcl,
                            clip1 );
                    }
                    if( !clip2.IsEmpty )
                    {
                        mgr.DrawThemeBackground( pe.Graphics,
                            (int)BUTTON_PARTS.BP_GROUPBOX,
                            (int)state,
                            rcl,
                            clip2 );
                    }
                    if( !clip3.IsEmpty )
                    {
                        mgr.DrawThemeBackground( pe.Graphics,
                            (int)BUTTON_PARTS.BP_GROUPBOX,
                            (int)state,
                            rcl,
                            clip3 );
                    }
                    if( Enabled )
                    {
                        using( Brush brush = new SolidBrush( ForeColor ) )
                        {
                            pe.Graphics.DrawString( Text, Font, brush, textRect, format );
                        }
                    }
                    else
                    {
                        ControlPaint.DrawStringDisabled( pe.Graphics, Text, Font, BackColor, textRect, format );
                    }
                    mgr.CloseThemeData();
                }
                catch( Exception e )
                {
                    MessageBox.Show( e.ToString() );
                    base.OnPaint(pe);
                }
            }
            else
            {
                base.OnPaint(pe);
            }
        }

        private StringFormat GetStringFormat()
        {
            //StringFormat format = new StringFormat();
            return StringFormat.GenericDefault;
        }
    }

    /// <summary>
    /// Paints a lable with a transparent background
    /// </summary>
    public class ThemedLabel : Label
    {
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedLabel()
        {
            SetStyle( ControlStyles.AllPaintingInWmPaint | 
                ControlStyles.SupportsTransparentBackColor, true );
            mgr = new XPThemeManager( this );
        }

        /// <summary>
        /// Paint background
        /// </summary>
        /// <param name="pe">Event args</param>
        protected override void OnPaintBackground( PaintEventArgs pe )
        {
            if( !XPThemeManager.IsAppThemed || BackColor.A == 255 )
            {
                base.OnPaintBackground( pe );
            }
            else
            {
                mgr.OpenThemeData( "BUTTON" );
                mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );
                mgr.CloseThemeData();
            }
        }

        /// <summary>
        /// Paint
        /// </summary>
        /// <param name="pe">event args</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            if( XPThemeManager.IsAppThemed )
            {
                OnPaintBackground( pe );
                try
                {
                    StringFormat format = GetStringFormat();
                    using( Brush brush = new SolidBrush( ForeColor ) )
                    {
                        if( Enabled )
                        {
                            pe.Graphics.DrawString( Text, Font, brush, ClientRectangle, format );
                        }
                        else
                        {
                            ControlPaint.DrawStringDisabled( pe.Graphics, Text, Font, BackColor, ClientRectangle, format );
                        }
                    }
                }
                catch( Exception )
                {
                    base.OnPaint(pe);
                }
            }
            else
            {
                Region reg = new System.Drawing.Region();
                reg.MakeInfinite();
                Region = reg;
                base.OnPaint(pe);
            }
        }

        private StringFormat GetStringFormat()
        {
            StringFormat format = new StringFormat();
            switch( this.TextAlign )
            {
                case ContentAlignment.BottomCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.BottomLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.BottomRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.MiddleCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.TopCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                default:
                    break;
            }
            return format;
        }
    }

    /// <summary>
    /// Paints a lable with a transparent background
    /// </summary>
    public class ThemedPanel : Panel
    {
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedPanel()
        {
            SetStyle( ControlStyles.AllPaintingInWmPaint | 
                ControlStyles.SupportsTransparentBackColor, true );
            mgr = new XPThemeManager( this );
        }

        /// <summary>
        /// Paint background
        /// </summary>
        /// <param name="pe">Event args</param>
        protected override void OnPaintBackground( PaintEventArgs pe )
        {
            if( !XPThemeManager.IsAppThemed || BackColor.A == 255 )
            {
                base.OnPaintBackground( pe );
            }
            else
            {
                mgr.OpenThemeData( "BUTTON" );
                mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );
                mgr.CloseThemeData();
            }
        }

        /// <summary>
        /// Paint
        /// </summary>
        /// <param name="pe">event args</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            if( !XPThemeManager.IsAppThemed )
            {
                base.OnPaint(pe);
            }
            else
            {
                OnPaintBackground( pe );
            }
        }
    }

    /// <summary>
    /// A standard radio button, but painted using themes
    /// and a transparent label.
    /// </summary>
    public class ThemedRadioButton : System.Windows.Forms.RadioButton
    {
        bool mouseDown = false;
        bool mouseOver = false;
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedRadioButton()
        {
            SetStyle( ControlStyles.AllPaintingInWmPaint | 
                ControlStyles.SupportsTransparentBackColor, true );
            mgr = new XPThemeManager( this );
        }

        /// <summary>
        /// Background paint method.
        /// </summary>
        /// <param name="pe">event args</param>
        protected override void OnPaintBackground( PaintEventArgs pe )
        {
            if( !XPThemeManager.IsAppThemed || BackColor.A == 255 )
            {
                base.OnPaintBackground( pe );
            }
            else
            {
                mgr.OpenThemeData( "BUTTON" );
                mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );
                mgr.CloseThemeData();
            }
        }

        /// <summary>
        /// Paint method
        /// </summary>
        /// <param name="pe">Event args</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            RADIOBUTTON_STATES state = RADIOBUTTON_STATES.RBS_CHECKEDNORMAL;

            if( XPThemeManager.IsAppThemed )
            {
                OnPaintBackground( pe );
                if( this.Checked )
                {
                    if( !Enabled )
                    {
                        state = RADIOBUTTON_STATES.RBS_CHECKEDDISABLED;
                    }
                    else if( mouseOver && mouseDown )
                    {
                        state = RADIOBUTTON_STATES.RBS_CHECKEDPRESSED;
                    }
                    else if( mouseOver || mouseDown )
                    {
                        state = RADIOBUTTON_STATES.RBS_CHECKEDHOT;
                    }
                    else
                    {
                        state = RADIOBUTTON_STATES.RBS_CHECKEDNORMAL;
                    }
                }
                else
                {
                    if( !Enabled )
                    {
                        state = RADIOBUTTON_STATES.RBS_UNCHECKEDDISABLED;
                    }
                    else if( mouseOver && mouseDown )
                    {
                        state = RADIOBUTTON_STATES.RBS_UNCHECKEDPRESSED;
                    }
                    else if( mouseOver || mouseDown )
                    {
                        state = RADIOBUTTON_STATES.RBS_UNCHECKEDHOT;
                    }
                    else
                    {
                        state = RADIOBUTTON_STATES.RBS_UNCHECKEDNORMAL;
                    }
                }

                try
                {
                    mgr.OpenThemeData( "BUTTON" );

                    mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );

                    Rectangle target = GetBackgroundRectangle(    mgr.GetThemePartSize( pe.Graphics, 
                        (int)BUTTON_PARTS.BP_RADIOBUTTON,
                        (int)state,
                        ClientRectangle, THEMESIZE.TS_DRAW ) );
                    mgr.DrawThemeBackground( pe.Graphics,
                        (int)BUTTON_PARTS.BP_RADIOBUTTON,
                        (int)state,
                        target,
                        pe.ClipRectangle );

                    Rectangle rcl = mgr.GetThemeBackgroundContentRect( pe.Graphics,
                        (int)BUTTON_PARTS.BP_RADIOBUTTON,
                        (int)state,
                        ClientRectangle );

                    if( ((CheckAlign == ContentAlignment.BottomLeft) ||
                        (CheckAlign == ContentAlignment.MiddleLeft) ||
                        (CheckAlign == ContentAlignment.TopLeft)) &&
                        ((TextAlign == ContentAlignment.BottomLeft) ||
                        (TextAlign == ContentAlignment.MiddleLeft) ||
                        (TextAlign == ContentAlignment.TopLeft)) )
                    {
                        rcl.X = target.Right + 1;
                    }

                    if( ((CheckAlign == ContentAlignment.BottomRight) ||
                        (CheckAlign == ContentAlignment.MiddleRight) ||
                        (CheckAlign == ContentAlignment.TopRight)) &&
                        ((TextAlign == ContentAlignment.BottomRight) ||
                        (TextAlign == ContentAlignment.MiddleRight) ||
                        (TextAlign == ContentAlignment.TopRight)) )
                    {
                        rcl.X -= target.Width + 1;
                    }

                    if( ((ImageAlign == ContentAlignment.BottomLeft) ||
                        (ImageAlign == ContentAlignment.MiddleLeft) ||
                        (ImageAlign == ContentAlignment.TopLeft)) &&
                        ((TextAlign == ContentAlignment.BottomLeft) ||
                        (TextAlign == ContentAlignment.MiddleLeft) ||
                        (TextAlign == ContentAlignment.TopLeft)) )
                    {
                        rcl.X += Image.Width + 1;
                    }

                    if( ((ImageAlign == ContentAlignment.BottomRight) ||
                        (ImageAlign == ContentAlignment.MiddleRight) ||
                        (ImageAlign == ContentAlignment.TopRight)) &&
                        ((TextAlign == ContentAlignment.BottomRight) ||
                        (TextAlign == ContentAlignment.MiddleRight) ||
                        (TextAlign == ContentAlignment.TopRight)) )
                    {
                        rcl.X -= Image.Width + 1;
                    }

                    StringFormat format = GetStringFormat();
                    using( Brush brush = new SolidBrush( ForeColor ) )
                    {
                        if( Enabled )
                        {
                            pe.Graphics.DrawString( Text, Font, brush, rcl, format );
                        }
                        else
                        {
                            ControlPaint.DrawStringDisabled( pe.Graphics, Text, Font, BackColor, rcl, format );
                        }
                    }
                    if( this.Image != null )
                    {
                        PaintImage( pe, (int)state, target );
                    }
                    mgr.CloseThemeData();
                }
                catch( Exception e )
                {
                    MessageBox.Show( e.ToString() );
                    base.OnPaint(pe);
                }
            }
            else
            {
                Region reg = new System.Drawing.Region();
                reg.MakeInfinite();
                Region = reg;
                base.OnPaint(pe);
            }
        }

        private void PaintImage( PaintEventArgs pe, int state, Rectangle target )
        {
            Rectangle destRect = mgr.GetThemeBackgroundContentRect( pe.Graphics,
                (int)BUTTON_PARTS.BP_RADIOBUTTON,
                state,
                ClientRectangle );
            switch( ImageAlign )
            {
                case ContentAlignment.BottomCenter:
                    destRect.Y = destRect.Bottom - Image.Height;
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.BottomLeft:
                    destRect.Y = destRect.Bottom - Image.Height;
                    break;
                case ContentAlignment.BottomRight:
                    destRect.Y = destRect.Bottom - Image.Height;
                    destRect.X = destRect.Right - Image.Width;
                    break;
                case ContentAlignment.MiddleCenter:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.MiddleLeft:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    break;
                case ContentAlignment.MiddleRight:
                    destRect.Y = destRect.Top + ((destRect.Height - Image.Height) / 2);
                    destRect.X = destRect.Right - Image.Width;
                    break;
                case ContentAlignment.TopCenter:
                    destRect.X = destRect.Left + ((destRect.Width - Image.Width) / 2);
                    break;
                case ContentAlignment.TopLeft:
                    break;
                case ContentAlignment.TopRight:
                    destRect.X = destRect.Right - Image.Width;
                    break;
                default:
                    break;
            }
        
            if( ((CheckAlign == ContentAlignment.BottomLeft) ||
                (CheckAlign == ContentAlignment.MiddleLeft) ||
                (CheckAlign == ContentAlignment.TopLeft)) &&
                ((ImageAlign == ContentAlignment.BottomLeft) ||
                (ImageAlign == ContentAlignment.MiddleLeft) ||
                (ImageAlign == ContentAlignment.TopLeft)) )
            {
                destRect.X += target.Width + 1;
            }

            if( ((CheckAlign == ContentAlignment.BottomRight) ||
                (CheckAlign == ContentAlignment.MiddleRight) ||
                (CheckAlign == ContentAlignment.TopRight)) &&
                ((ImageAlign == ContentAlignment.BottomRight) ||
                (ImageAlign == ContentAlignment.MiddleRight) ||
                (ImageAlign == ContentAlignment.TopRight)) )
            {
                destRect.X -= target.Width + 1;
            }
            pe.Graphics.DrawImageUnscaled( this.Image, destRect );
        }

        protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs mevent)
        {
            mouseDown = false;
            base.OnMouseUp( mevent );
        }

        protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs mevent)
        {
            mouseDown = true;
            base.OnMouseDown( mevent );
        }

        protected override void OnMouseEnter(System.EventArgs eventargs)
        {
            mouseOver = true;
            base.OnMouseEnter( eventargs );
            Invalidate();
        }

        protected override void OnMouseLeave(System.EventArgs eventargs)
        {
            mouseOver = false;
            base.OnMouseLeave( eventargs );
            Invalidate();
        }

        private StringFormat GetStringFormat()
        {
            StringFormat format = new StringFormat();
            switch( this.TextAlign )
            {
                case ContentAlignment.BottomCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.BottomLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.BottomRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Far;
                    break;
                case ContentAlignment.MiddleCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.MiddleRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Center;
                    break;
                case ContentAlignment.TopCenter:
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopLeft:
                    format.Alignment = StringAlignment.Near;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                case ContentAlignment.TopRight:
                    format.Alignment = StringAlignment.Far;
                    format.LineAlignment = StringAlignment.Near;
                    break;
                default:
                    break;
            }
            return format;
        }

        private Rectangle GetBackgroundRectangle( Size size )
        {
            Rectangle destRect = new Rectangle( new Point( 0,0 ), size );
            switch( this.CheckAlign )
            {
                case ContentAlignment.BottomCenter:
                    destRect.Y = ClientRectangle.Bottom - size.Height;
                    destRect.X = destRect.Left + ((ClientRectangle.Width - size.Width) / 2);
                    break;
                case ContentAlignment.BottomLeft:
                    destRect.Y = ClientRectangle.Bottom - size.Height;
                    break;
                case ContentAlignment.BottomRight:
                    destRect.Y = ClientRectangle.Bottom - size.Height;
                    destRect.X = ClientRectangle.Right - size.Width;
                    break;
                case ContentAlignment.MiddleCenter:
                    destRect.Y = destRect.Top + ((ClientRectangle.Height - size.Height) / 2);
                    destRect.X = destRect.Left + ((ClientRectangle.Width - size.Width) / 2);
                    break;
                case ContentAlignment.MiddleLeft:
                    destRect.Y = destRect.Top + ((ClientRectangle.Height - size.Height) / 2);
                    break;
                case ContentAlignment.MiddleRight:
                    destRect.Y = destRect.Top + ((ClientRectangle.Height - size.Height) / 2);
                    destRect.X = ClientRectangle.Right - size.Width;
                    break;
                case ContentAlignment.TopCenter:
                    destRect.X = destRect.Left + ((ClientRectangle.Width - size.Width) / 2);
                    break;
                case ContentAlignment.TopLeft:
                    break;
                case ContentAlignment.TopRight:
                    destRect.X = ClientRectangle.Right - size.Width;
                    break;
                default:
                    break;
            }
            return destRect;
        }

        /// <summary>
        /// Mouse handler
        /// </summary>
        /// <param name="mevent">Event args</param>
        protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs mevent)
        {
            base.OnMouseMove( mevent );
            if( this.ClientRectangle.Contains( mevent.X, mevent.Y ) )
            {
                if( !mouseOver )
                {
                    mouseOver = true;
                    Invalidate();
                }
            }
            else
            {
                if( mouseOver )
                {
                    mouseOver = false;
                    Invalidate();
                }
            }
        }
    }

    /// <summary>
    /// Paints a themed button
    /// </summary>
    public class ThemedTabPage : System.Windows.Forms.TabPage
    {        
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedTabPage()
        {
            SetStyle( ControlStyles.ResizeRedraw, true );
            mgr = new XPThemeManager( this );
        }

        protected override void OnPaintBackground(System.Windows.Forms.PaintEventArgs e)
        {
            if (XPThemeManager.IsAppThemed)
            {
                mgr.OpenThemeData( "TAB" );
                mgr.DrawThemeBackground(
                    e.Graphics, 
                    (int)TAB_PARTS.TABP_BODY, 
                    (int)TABITEM_STATES.TIS_NORMAL, 
                    ClientRectangle,
                    e.ClipRectangle );
                mgr.CloseThemeData();
            }
            else
            {
                base.OnPaintBackground (e);
            }
        }
    }

    /// <summary>
    /// Paints a lable with a transparent background
    /// </summary>
    public class ThemedUserControl : UserControl
    {
        XPThemeManager mgr;

        /// <summary>
        /// Standard constructor
        /// </summary>
        public ThemedUserControl()
        {
            SetStyle( ControlStyles.AllPaintingInWmPaint | 
                ControlStyles.SupportsTransparentBackColor, true );
            mgr = new XPThemeManager( this );
        }

        /// <summary>
        /// Paint background
        /// </summary>
        /// <param name="pe">Event args</param>
        protected override void OnPaintBackground( PaintEventArgs pe )
        {
            if( !XPThemeManager.IsAppThemed || BackColor.A == 255 )
            {
                base.OnPaintBackground( pe );
            }
            else
            {
                mgr.OpenThemeData( "BUTTON" );
                mgr.DrawThemeParentBackground( pe.Graphics, ClientRectangle );
                mgr.CloseThemeData();
            }
        }

        /// <summary>
        /// Paint
        /// </summary>
        /// <param name="pe">event args</param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            if( !XPThemeManager.IsAppThemed )
            {
                base.OnPaint(pe);
            }
            else
            {
                OnPaintBackground( pe );
            }
        }
    }

    /// <summary>
    /// Base exception thrown by XP theme manager
    /// </summary>
    public class XPThemeException : ApplicationException {}

    /// <summary>
    /// An XP Theme Manager class for .NET
    /// </summary>
    public class XPThemeManager : IDisposable
    {
        IntPtr themeHandle = IntPtr.Zero;
        Control activeControl = null;
        string activeClassList = "";
        
        /// <summary>
        /// Standard constructor
        /// </summary>
        /// <param name="control">The control to theme</param>
        public XPThemeManager( Control control )
        {
            activeControl = control;
            AttachToControl( control );
        }

        public static Color AlternatingColor
        {
            get
            {
                return Color.FromArgb(
                    (int)(SystemColors.Window.R * 0.90 + SystemColors.Highlight.G * 0.1),
                    (int)(SystemColors.Window.G * 0.90 + SystemColors.Highlight.G * 0.1),
                    (int)(SystemColors.Window.B * 0.90 + SystemColors.Highlight.B * 0.1) );
            }
        }

        /// <summary>
        /// Gets a value indicating whether the application is themed
        /// </summary>
        public static bool IsAppThemed
        {
            get
            {
                try
                {
                    return XPThemeWrapper.IsAppThemed();
                }
                catch( Exception )
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets a value whcih determines if theming is active
        /// </summary>
        public static bool IsThemeActive
        {
            get
            {
                try
                {
                    return XPThemeWrapper.IsThemeActive();
                }
                catch( Exception )
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets a value which determines whether this manager
        /// has an open theme.
        /// </summary>
        public bool HasOpenTheme
        {
            get
            {
                return( themeHandle != IntPtr.Zero );
            }
        }

        /// <summary>
        /// Opens a theme
        /// </summary>
        /// <param name="classList">The list of theme classes to open</param>
        public void OpenThemeData( string classList )
        {
            OpenTheme( classList );
            activeClassList = classList;
        }

        /// <summary>
        /// Closes an open theme
        /// </summary>
        public void CloseThemeData()
        {
            CloseTheme();
            activeClassList = "";
        }

        /// <summary>
        /// Draws the background area for the theme
        /// </summary>
        /// <param name="graphics">The surface on which to draw</param>
        /// <param name="partId">The part to draw (see schemadefs.h)</param>
        /// <param name="stateId">The state to draw (see schemadefs.h)</param>
        /// <param name="targetRect">The target rectangle</param>
        /// <param name="clip">The clip rectangle</param>
        public void DrawThemeBackground( Graphics graphics, int partId, int stateId,
            Rectangle targetRect, Rectangle clip )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            RECT tgt = XPThemeWrapper.FromRectangle( targetRect );
            RECT clp = XPThemeWrapper.FromRectangle( clip );
            if( XPThemeWrapper.DrawThemeBackground( themeHandle, hdc, partId, stateId,
                ref tgt,
                ref clp ) != 0 )
            {
                throw new XPThemeException();
            }
            GC.KeepAlive( tgt );
            GC.KeepAlive( clp );
            GC.KeepAlive( hdc );
            graphics.ReleaseHdc( hdc );
        }

        /// <summary>
        /// Gets the region corresponding to a theme background
        /// </summary>
        /// <param name="graphics">The graphics surface</param>
        /// <param name="partId">The part to measure (see schemadefs.h)</param>
        /// <param name="stateId">The state of the part (see schemadefs.h)</param>
        /// <param name="targetRect">The target rectangle for the background</param>
        /// <returns>A region representing the background area</returns>
        public Region GetThemeBackgroundRegion( Graphics graphics, int partId, int stateId,
            Rectangle targetRect )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            RECT tgt = XPThemeWrapper.FromRectangle( targetRect );

            IntPtr region = IntPtr.Zero;

            if( XPThemeWrapper.GetThemeBackgroundRegion( themeHandle, hdc, partId, stateId, ref tgt, ref region ) != 0 )
            {
                throw new XPThemeException();
            }

            GC.KeepAlive( tgt );
            GC.KeepAlive( hdc );
            GC.KeepAlive( region );
            graphics.ReleaseHdc( hdc );

            return Region.FromHrgn( region );
        }

        /// <summary>
        /// Gets the text extent for a particular
        /// string when rendered using the theme manager
        /// </summary>
        /// <param name="graphics">The graphics surface on which to draw</param>
        /// <param name="partId">The part to draw (see schemadefs.h)</param>
        /// <param name="stateId">The state to draw (see schemadefs.h)</param>
        /// <param name="text">The text to measure</param>
        /// <param name="format">The format of the text</param>
        /// <returns>The bounding rectangle for the text on the surface</returns>
        public Rectangle GetThemeTextExtent( Graphics graphics, int partId, int stateId,
            string text, StringFormat format )
        {
            return GetThemeTextExtent( graphics, partId, stateId, text, format, Rectangle.Empty );
        }

        /// <summary>
        /// Gets the text extent for a particular
        /// string when rendered using the theme manager
        /// </summary>
        /// <param name="graphics">The graphics surface on which to draw</param>
        /// <param name="partId">The part to draw (see schemadefs.h)</param>
        /// <param name="stateId">The state to draw (see schemadefs.h)</param>
        /// <param name="text">The text to measure</param>
        /// <param name="format">The format of the text</param>
        /// <param name="bounds">The clip bounds for the text</param>
        /// <returns>The bounding rectangle for the text on the surface</returns>
        public Rectangle GetThemeTextExtent( Graphics graphics, int partId, int stateId,
            string text, StringFormat format, Rectangle bounds )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            RECT bnds = XPThemeWrapper.FromRectangle( bounds );
            int fmt = GetFlagsFor( format );
            RECT extent = new RECT();

            int rc = 0;
            if( bounds.IsEmpty )
            {
                rc = XPThemeWrapper.GetThemeTextExtent( themeHandle, hdc, partId, stateId, text, text.Length, fmt, IntPtr.Zero, ref extent );
            }
            else
            {
                rc = XPThemeWrapper.GetThemeTextExtent( themeHandle, hdc, partId, stateId, text, text.Length, fmt, ref bnds, ref extent );
            }

            if( rc != 0 )
            {
                throw new XPThemeException();
            }

            GC.KeepAlive( bnds );
            GC.KeepAlive( hdc );
            GC.KeepAlive( extent );
            graphics.ReleaseHdc( hdc );

            return XPThemeWrapper.FromRECT( extent );
        }

        /// <summary>
        /// Draws the text on the graphics surface
        /// </summary>
        /// <param name="graphics">The graphics surface on which to draw</param>
        /// <param name="partId">The part to draw (see schemadefs.h)</param>
        /// <param name="stateId">The state to draw (see schemadefs.h)</param>
        /// <param name="text">The text to measure</param>
        /// <param name="format">The format of the text</param>
        /// <param name="bounds">The clip bounds for the text</param>
        public void DrawThemeText( Graphics graphics, int partId, int stateId,
            string text, StringFormat format, bool enabled, Rectangle bounds )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            RECT bnds = XPThemeWrapper.FromRectangle( bounds );
            int fmt = GetFlagsFor( format );

            if(  XPThemeWrapper.DrawThemeText( themeHandle, hdc, partId, stateId, text, text.Length, fmt, enabled ? 0:1, ref bnds ) != 0 )
            {
                throw new XPThemeException();
            }

            GC.KeepAlive( bnds );
            GC.KeepAlive( hdc );
            graphics.ReleaseHdc( hdc );
        }

        /// <summary>
        /// Gets the rectangle in which the content imagery should
        /// be contained.
        /// </summary>
        /// <param name="graphics">The graphics surface on which to draw</param>
        /// <param name="partId">The part to draw (see schemadefs.h)</param>
        /// <param name="stateId">The state to draw (see schemadefs.h)</param>
        /// <param name="bounds">The bounding rectangle for the part</param>
        /// <returns>The bounding rectangle for the part content</returns>
        public Rectangle GetThemeBackgroundContentRect( Graphics graphics, 
            int partId, int stateId, Rectangle bounds )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            RECT bnds = XPThemeWrapper.FromRectangle( bounds );
            RECT extent = new RECT();

            if( XPThemeWrapper.GetThemeBackgroundContentRect( themeHandle, hdc, partId, stateId, ref bnds, ref extent ) != 0 )
            {
                throw new XPThemeException();
            }

            GC.KeepAlive( bnds );
            GC.KeepAlive( hdc );
            GC.KeepAlive( extent );
            graphics.ReleaseHdc( hdc );

            return XPThemeWrapper.FromRECT( extent );
        }

        /// <summary>
        /// Gets the rectangle bounding the theme background
        /// </summary>
        /// <param name="graphics">The graphics surface on which to draw</param>
        /// <param name="partId">The part to draw (see schemadefs.h)</param>
        /// <param name="stateId">The state to draw (see schemadefs.h)</param>
        /// <param name="bounds">The bounds of the themed object</param>
        /// <returns>The bounding rectangle of the background of the themed object</returns>
        public Rectangle GetThemeBackgroundExtent( Graphics graphics, 
            int partId, int stateId, Rectangle bounds )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            RECT bnds = XPThemeWrapper.FromRectangle( bounds );
            RECT extent = new RECT();

            if( XPThemeWrapper.GetThemeBackgroundExtent( themeHandle, hdc, partId, stateId, ref bnds, ref extent ) != 0 )
            {
                throw new XPThemeException();
            }

            GC.KeepAlive( bnds );
            GC.KeepAlive( hdc );
            GC.KeepAlive( extent );
            graphics.ReleaseHdc( hdc );

            return XPThemeWrapper.FromRECT( extent );
        }

        /// <summary>
        /// Returns the size of a particular element of a part on a graphics surface
        /// </summary>
        /// <param name="graphics">The graphics surface on which to draw</param>
        /// <param name="partId">The part to draw (see schemadefs.h)</param>
        /// <param name="stateId">The state to draw (see schemadefs.h)</param>
        /// <param name="bounds">The bounds of the part</param>
        /// <param name="size">The size to return (see schemadefs.h)</param>
        /// <returns>The size of the requested part element</returns>
        public Size GetThemePartSize( Graphics graphics,
            int partId, int stateId, Rectangle bounds, THEMESIZE size )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            RECT bnds = XPThemeWrapper.FromRectangle( bounds );
            SIZE extent = new SIZE();

            if( XPThemeWrapper.GetThemePartSize( themeHandle, hdc, partId, stateId, ref bnds, (int)size, ref extent ) != 0 )
            {
                throw new XPThemeException();
            }

            GC.KeepAlive( bnds );
            GC.KeepAlive( hdc );
            GC.KeepAlive( extent );
            graphics.ReleaseHdc( hdc );

            return XPThemeWrapper.FromSIZE( extent );
        }

        /// <summary>
        /// Causes the parent to draw its background on the part's
        /// graphics surface. This can be used to generate pseudo-transparent
        /// elements.
        /// </summary>
        /// <param name="graphics">The graphics surface on which to draw</param>
        /// <param name="bounds">The bounding rectangle in which the parent
        /// background should be rendered.
        /// </param>
        public void DrawThemeParentBackground( Graphics graphics, Rectangle bounds )
        {
            if( !HasOpenTheme )
            {
                throw new XPThemeException();
            }
            IntPtr hdc = graphics.GetHdc();
            IntPtr handle = activeControl.Handle;

            RECT bnds = XPThemeWrapper.FromRectangle( bounds );
    
            if( XPThemeWrapper.DrawThemeParentBackground( handle, hdc, ref bnds ) != 0 )
            {
                throw new XPThemeException();
            }

            GC.KeepAlive( bnds );
            GC.KeepAlive( hdc );
            GC.KeepAlive( handle );
            graphics.ReleaseHdc( hdc );
        }

        /// <summary>
        /// Paints a particular style of gradient background which fades from a theme-defined color at the top,
        /// to a specified color at the base (e.g. White).
        /// </summary>
        /// <param name="rcl">The total target rectangle of the Window.</param>
        /// <param name="g">The graphics surface on which to paint the fill.</param>
        /// <param name="clip">The current clip rectangle for the surface</param>
        /// <param name="sourceColor">The color to which to blend.</param>
        public static void DrawGraduatedWizardBackground( Rectangle rcl, System.Drawing.Graphics g, Rectangle clip, Color sourceColor )
        {
            if( !clip.IsEmpty )
            {
                System.Drawing.Color baseColor = SystemColors.ActiveCaption;
                System.Drawing.Color color = Color.FromArgb( (int)Math.Min( 255f, (baseColor.R + SystemColors.Window.R) / 2.5f ), (int)Math.Min( 255f, (baseColor.G + SystemColors.Window.G) / 2.5f ), (int)Math.Min( 255f, (baseColor.B + SystemColors.Window.B) / 2.5f) );
                using( LinearGradientBrush brush = new LinearGradientBrush( rcl, color, sourceColor, LinearGradientMode.Vertical ) )
                {
                    g.FillRectangle( brush, clip );
                }
            }
        }

        /// <summary>
        /// Paints a diagonal graduated background into a control, based on the current Theme colors.
        /// </summary>
        /// <param name="control">The control whose background needs painting</param>
        /// <param name="g">The graphics surface on which to paint.</param>
        /// <param name="clip">The clip rectangle to apply.</param>
        public static void DrawGraduatedWizardBackground( Control control, System.Drawing.Graphics g, Rectangle clip )
        {
            Rectangle rcl = new Rectangle( 0, 0, control.Width, control.Height );
            if( !rcl.IsEmpty )
            {
                System.Drawing.Color baseColor = SystemColors.ActiveCaption;
                System.Drawing.Color color1 = SystemColors.InactiveCaption;
                System.Drawing.Color color2 = Color.FromArgb( (int)Math.Min( 255f, (baseColor.R + 255f) / 2.5f ), (int)Math.Min( 255f, (baseColor.G + 255f) / 2.5f ), (int)Math.Min( 255f, (baseColor.B + 255f) / 2.5f) );
                using( LinearGradientBrush brush = new LinearGradientBrush( rcl, color1, color2, LinearGradientMode.ForwardDiagonal ) )
                {
                    g.FillRectangle( brush, clip );
                }
            }
        }


        #region Event Handlers
        private void ControlColorsChanged( object sender, EventArgs e )
        {
            if( HasOpenTheme )
            {
                CloseTheme();
            }
            if( IsAppThemed && (activeClassList != "") )
            {
                OpenTheme( activeClassList );
            }
        }

        private void ControlHandleDestroyed( object sender, EventArgs e )
        {
            if( HasOpenTheme )
            {
                CloseTheme();
            }
        }

        private void ControlHandleCreated( object sender, EventArgs e )
        {
            if( IsAppThemed && (activeClassList != "") )
            {
                OpenTheme( activeClassList );
            }
        }
        #endregion

        #region Implementation Utilities
        private void AttachToControl( Control control )
        {
            control.SystemColorsChanged += new EventHandler( ControlColorsChanged );
            control.HandleDestroyed += new EventHandler( ControlHandleDestroyed );
            control.HandleCreated += new EventHandler( ControlHandleCreated );
        }

        private void DetachFromControl( Control control )
        {
            control.SystemColorsChanged -= new EventHandler( ControlColorsChanged );
            control.HandleDestroyed -= new EventHandler( ControlHandleDestroyed );
            control.HandleCreated -= new EventHandler( ControlHandleCreated );
        }

        private void CloseTheme()
        {
            if( themeHandle == IntPtr.Zero )
            {
                throw new XPThemeException();
            }
            if( XPThemeWrapper.CloseThemeData( themeHandle ) != 0 )
            {
                throw new XPThemeException();
            }
            GC.KeepAlive( themeHandle );
            themeHandle = IntPtr.Zero;
        }

        private void OpenTheme( string className )
        {
            if( themeHandle != IntPtr.Zero )
            {
                throw new XPThemeException();
            }
            if( activeControl.Handle == IntPtr.Zero )
            {
                throw new XPThemeException();
            }
            themeHandle = XPThemeWrapper.OpenThemeData( activeControl.Handle, className );
            GC.KeepAlive( activeControl.Handle );
            GC.KeepAlive( className );
            if( themeHandle == IntPtr.Zero )
            {
                throw new XPThemeException();
            }
        }

        private int GetFlagsFor( StringFormat format )
        {
            int fmt = 0;
            switch( format.Alignment )
            {
                case StringAlignment.Center:
                    fmt += (int)TextFormatFlags.DT_CENTER;
                    break;
                case StringAlignment.Far:
                    fmt += (int)TextFormatFlags.DT_RIGHT;
                    break;
                case StringAlignment.Near:
                    fmt += (int)TextFormatFlags.DT_LEFT;
                    break;
                default:
                    break;
            }
            if( (format.FormatFlags & StringFormatFlags.DirectionRightToLeft ) != 0 )
            {
                fmt += (int)TextFormatFlags.DT_RTLREADING;
            }

            if( (format.FormatFlags & StringFormatFlags.NoClip ) != 0 )
            {
                fmt += (int)TextFormatFlags.DT_NOCLIP;
            }

            if( (format.FormatFlags & StringFormatFlags.NoWrap ) != 0 )
            {
                fmt += (int)TextFormatFlags.DT_SINGLELINE;
            }

            switch(format.LineAlignment)
            {
                case StringAlignment.Center:
                    fmt += (int)TextFormatFlags.DT_VCENTER;
                    break;
                case StringAlignment.Far:
                    fmt += (int)TextFormatFlags.DT_BOTTOM;
                    break;
                case StringAlignment.Near:
                    fmt += (int)TextFormatFlags.DT_TOP;
                    break;
                default:
                    break;
            }

            switch( format.Trimming )
            {
                case StringTrimming.EllipsisCharacter:
                    fmt += (int)TextFormatFlags.DT_END_ELLIPSIS;
                    break;
                case StringTrimming.EllipsisPath:
                    fmt += (int)TextFormatFlags.DT_PATH_ELLIPSIS;
                    break;
                case StringTrimming.EllipsisWord:
                    fmt += (int)TextFormatFlags.DT_WORD_ELLIPSIS;
                    break;
                case StringTrimming.Word:
                    fmt += (int)TextFormatFlags.DT_WORDBREAK;
                    break;
                default:
                    break;
            }
            return fmt;
        }
        #endregion

        #region Implementation of IDisposable
        public void Dispose()
        {
            if( HasOpenTheme )
            {
                CloseTheme();
            }
            if( activeControl != null && !activeControl.IsDisposed )
            {
                DetachFromControl( activeControl );
            }
        }
        #endregion
    }

    [StructLayout(LayoutKind.Explicit)]
    internal struct RECT 
    {
        [FieldOffset(0)] public int left;
        [FieldOffset(4)] public int top;
        [FieldOffset(8)] public int right;
        [FieldOffset(12)] public int bottom;
    }   

    [StructLayout(LayoutKind.Explicit)]
    internal struct SIZE 
    {
        [FieldOffset(0)] public int width;
        [FieldOffset(4)] public int height;
    }   

    internal enum TextFormatFlags
    {
        DT_TOP                      = 0x00000000
            ,DT_LEFT                     = 0x00000000
                 ,DT_CENTER                   = 0x00000001
                      ,DT_RIGHT                    = 0x00000002
                           ,DT_VCENTER                  = 0x00000004
                                ,DT_BOTTOM                   = 0x00000008
                                     ,DT_WORDBREAK                = 0x00000010
                                          ,DT_SINGLELINE               = 0x00000020
                                               ,DT_EXPANDTABS               = 0x00000040
                                                    ,DT_TABSTOP                  = 0x00000080
                                                         ,DT_NOCLIP                   = 0x00000100
                                                              ,DT_EXTERNALLEADING          = 0x00000200
                                                                   ,DT_CALCRECT                 = 0x00000400
                                                                        ,DT_NOPREFIX                 = 0x00000800
                                                                             ,DT_INTERNAL                 = 0x00001000
                                                                                  ,DT_EDITCONTROL              = 0x00002000
                                                                                       ,DT_PATH_ELLIPSIS            = 0x00004000
                                                                                            ,DT_END_ELLIPSIS             = 0x00008000
                                                                                                 ,DT_MODIFYSTRING             = 0x00010000
                                                                                                      ,DT_RTLREADING               = 0x00020000
                                                                                                           ,DT_WORD_ELLIPSIS            = 0x00040000
                                                                                                                ,DT_NOFULLWIDTHCHARBREAK     = 0x00080000
                                                                                                                     ,DT_HIDEPREFIX               = 0x00100000
                                                                                                                          ,DT_PREFIXONLY               = 0x00200000
    }

    internal class XPThemeWrapper
    {
        internal static Rectangle FromRECT( RECT rect )
        {
            return Rectangle.FromLTRB( rect.left, rect.top, rect.right, rect.bottom );
        }

        internal static RECT FromRectangle( Rectangle rcl )
        {
            RECT rc = new RECT();
            rc.left = rcl.Left;
            rc.top = rcl.Top;
            rc.right = rcl.Right;
            rc.bottom = rcl.Bottom;
            return rc;
        }

        internal static Size FromSIZE( SIZE sz )
        {
            return new Size( sz.width, sz.height );
        }

        internal static SIZE FromSize( Size sz )
        {
            SIZE rc = new SIZE();
            rc.width = sz.Width;
            rc.height = sz.Height;
            return rc;
        }

        [DllImport("uxtheme.dll")]
        internal static extern IntPtr OpenThemeData( IntPtr hwnd, 
            [MarshalAs(UnmanagedType.LPWStr)]
            string pszClassList);
        [DllImport("uxtheme.dll")]
        internal static extern int CloseThemeData( IntPtr htheme );
        [DllImport("uxtheme.dll")]
        internal static extern bool IsAppThemed();
        [DllImport("uxtheme.dll")]
        internal static extern bool IsThemeActive();
        [DllImport("uxtheme.dll")]
        internal static extern int DrawThemeBackground( IntPtr hTheme, IntPtr hdc, 
            int iPartId, int iStateId,
            ref RECT pRect,
            ref RECT pClipRect);
        [DllImport("uxtheme.dll")]
        internal static extern int GetThemeBackgroundRegion(IntPtr hTheme, IntPtr hdc,  
            int iPartId, int iStateId,
            ref RECT pRect,
            ref IntPtr pRegion);
        [DllImport("uxtheme.dll")]
        internal static extern int GetThemeTextExtent(IntPtr hTheme, IntPtr hdc, 
            int iPartId, int iStateId,
            [MarshalAs(UnmanagedType.LPWStr)]
            string pszText, int iCharCount, 
            int dwTextFlags, ref RECT pBoundingRect, 
            ref RECT pExtentRect);

        [DllImport("uxtheme.dll")]
        internal static extern int GetThemeTextExtent(IntPtr hTheme, IntPtr hdc, 
            int iPartId, int iStateId,
            [MarshalAs(UnmanagedType.LPWStr)]
            string pszText, int iCharCount, 
            int dwTextFlags, IntPtr pBoundingRect, 
            ref RECT pExtentRect);

        [DllImport("uxtheme.dll")]
        internal static extern int GetThemeBackgroundContentRect(IntPtr hTheme, IntPtr hdc, 
            int iPartId, int iStateId,
            ref RECT pBoundingRect, 
            ref RECT pContentRect);

        [DllImport("uxtheme.dll")]
        internal static extern int DrawThemeText(IntPtr hTheme, IntPtr hdc,
            int iPartId, int iStateId,
            [MarshalAs(UnmanagedType.LPWStr)]
            string pszText, int iCharCount, int dwTextFlags, 
            int dwTextFlags2, ref RECT pRect);

        [DllImport("uxtheme.dll")]
        internal static extern int DrawThemeParentBackground(IntPtr hwnd, IntPtr hdc, ref RECT bounds );

        [DllImport("uxtheme.dll")]
        internal static extern int GetThemeBackgroundExtent(IntPtr hTheme, IntPtr hdc,
            int iPartId, int iStateId,
            ref RECT pContentRect, 
            ref RECT pExtentRect);

        [DllImport("uxtheme.dll")]
        internal static extern int GetThemePartSize(IntPtr hTheme, IntPtr hdc,
            int iPartId, int iStateId, 
            ref RECT prc, int eSize, ref SIZE psz);

    }

}


